@@ -1,7 +1,7 @@
 /* ay.h: AY-8-3912 routines
-   Copyright (c) 1999-2003 Philip Kendall
+   Copyright (c) 1999-2004 Philip Kendall
 
-   $Id: ay.h,v 1.11 2003/07/11 11:03:25 pak21 Exp $
+   $Id: ay.h,v 1.15 2004/06/15 09:48:40 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,15 +29,19 @@
 
 #include <libspectrum.h>
 
+#define AY_REGISTERS 16
+
 typedef struct ayinfo {
-  int present;
   int current_register;
-  libspectrum_byte registers[16];
+  libspectrum_byte registers[ AY_REGISTERS ];
 } ayinfo;
 
-libspectrum_byte ay_registerport_read( libspectrum_word port );
+libspectrum_byte ay_registerport_read( libspectrum_word port, int *attached );
 void ay_registerport_write( libspectrum_word port, libspectrum_byte b );
 
 void ay_dataport_write( libspectrum_word port, libspectrum_byte b );
 
+int ay_from_snapshot( libspectrum_snap *snap, int capabilities );
+int ay_to_snapshot( libspectrum_snap *snap );
+
 #endif			/* #ifndef FUSE_AY_H */
