@@ -1,7 +1,7 @@
 /* dck.c: dock snapshot (Warajevo .DCK) handling routines
-   Copyright (c) 2003 Darren Salt, Fredrick Meunier, Philip Kendall
+   Copyright (c) 2003-2004 Darren Salt, Fredrick Meunier, Philip Kendall
 
-   $Id: dck.c,v 1.12.2.1 2004/01/26 22:46:25 pak21 Exp $
+   $Id: dck.c,v 1.21 2004/05/25 10:05:05 fredm Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,24 +37,31 @@
 
 #include "dck.h"
 #include "machine.h"
+#include "memory.h"
 #include "settings.h"
 #include "scld.h"
-#include "tc2068.h"
 #include "ui/ui.h"
 #include "utils.h"
 #include "debugger/debugger.h"
 #include "widget/widget.h"
 
+/* Dock cart inserted? */
+int dck_active = 0;
+
 int
 dck_insert( const char *filename )
 {
   int error;
 
+  if ( !( libspectrum_machine_capabilities( machine_current->machine ) &
+	  LIBSPECTRUM_MACHINE_CAPABILITY_TIMEX_DOCK ) ) {
+    ui_error( UI_ERROR_ERROR, "This machine does not support the dock" );
+    return 1;
+  }
+
   error = settings_set_string( &settings_current.dck_file, filename );
   if( error ) return error;
 
-  error = dck_read( filename ); if( error ) return error;
-
   machine_reset();
 
   return 0;
@@ -72,34 +79,35 @@ dck_eject( void )
   if( settings_current.dck_file ) free( settings_current.dck_file );
   settings_current.dck_file = NULL;
 
-  ui_menu_activate( UI_MENU_ITEM_MEDIA_CARTRIDGE_EJECT, 0 );
+  dck_active = 0;
+
+  ui_menu_activate( UI_MENU_ITEM_MEDIA_CARTRIDGE_DOCK_EJECT, 0 );
 
   machine_reset();
 }
 
 int
-dck_read( const char *filename )
+dck_reset( void )
 {
   utils_file file;
   size_t num_block = 0;
   libspectrum_dck *dck;
   int error;
 
-  int i;
-  memory_page *mem;
+  dck_active = 0;
 
-  if ( !( libspectrum_machine_capabilities( machine_current->machine ) &
-	  LIBSPECTRUM_MACHINE_CAPABILITY_TIMEX_DOCK ) ) {
-    ui_error( UI_ERROR_ERROR, "This machine does not support the dock" );
-    return 1;
+  if( !settings_current.dck_file ) {
+    ui_menu_activate( UI_MENU_ITEM_MEDIA_CARTRIDGE_DOCK_EJECT, 0 );
+    return 0;
   }
 
   error = libspectrum_dck_alloc( &dck ); if( error ) return error;
 
-  error = utils_read_file( filename, &file );
+  error = utils_read_file( settings_current.dck_file, &file );
   if( error ) { libspectrum_dck_free( dck, 0 ); return error; }
 
-  error = libspectrum_dck_read2( dck, file.buffer, file.length, filename );
+  error = libspectrum_dck_read2( dck, file.buffer, file.length,
+                                 settings_current.dck_file );
   if( error ) {
     utils_close_file( &file ); libspectrum_dck_free( dck, 0 ); return error;
   }
@@ -110,15 +118,18 @@ dck_read( const char *filename )
   }
 
   while( dck->dck[num_block] != NULL ) {
+    memory_page **mem;
+    int i;
+
     switch( dck->dck[num_block]->bank ) {
     case LIBSPECTRUM_DCK_BANK_HOME:
-      mem = timex_home;
+      mem = memory_map_home;
       break;
     case LIBSPECTRUM_DCK_BANK_DOCK:
-      mem = timex_dock;
+      mem = memory_map_dock;
       break;
     case LIBSPECTRUM_DCK_BANK_EXROM:
-      mem = timex_exrom;
+      mem = memory_map_exrom;
       break;
     default:
       ui_error( UI_ERROR_INFO, "Sorry, bank ID %i is unsupported",
@@ -128,33 +139,37 @@ dck_read( const char *filename )
     }
 
     for( i = 0; i < 8; i++ ) {
+
       switch( dck->dck[num_block]->access[i] ) {
 
       case LIBSPECTRUM_DCK_PAGE_NULL:
         break;
 
       case LIBSPECTRUM_DCK_PAGE_ROM:
-        mem[i].page = dck->dck[num_block]->pages[i];
-        mem[i].allocated = 1;
-        mem[i].writable = 0;
-	mem[i].reverse = -1;
+        mem[i]->page = memory_pool_allocate( MEMORY_PAGE_SIZE );
+	if( !mem[i]->page ) return 1;
+	memcpy( mem[i]->page, dck->dck[num_block]->pages[i],
+		MEMORY_PAGE_SIZE );
+        mem[i]->writable = 0;
+        mem[i]->source = MEMORY_SOURCE_CARTRIDGE;
         break;
 
       case LIBSPECTRUM_DCK_PAGE_RAM_EMPTY:
       case LIBSPECTRUM_DCK_PAGE_RAM:
-        if( dck->dck[num_block]->bank == LIBSPECTRUM_DCK_BANK_HOME && i > 1 ) {
-          /* Because the scr and snapshot code depends on the standard memory
-             map being in the RAM[] array, we copy blocks from the HOME bank
-             into the appropriate page via the timex_home structure */
-          memcpy( mem[i].page, dck->dck[num_block]->pages[i], 0x2000 );
-          free( dck->dck[num_block]->pages[i] );
-        } else {
-          mem[i].page = dck->dck[num_block]->pages[i];
-          mem[i].allocated = 1;
-          mem[i].writable = 1;
-	  mem[i].reverse = -1;
+	/* Because the scr and snapshot code depends on the standard
+	   memory map being in the RAM[] array, we just copy RAM
+	   blocks from the HOME bank into the appropriate page; in
+	   other cases, we allocate ourselves a new page to store the
+	   contents in */
+        if( !(dck->dck[num_block]->bank == LIBSPECTRUM_DCK_BANK_HOME && i>1) ) {
+          mem[i]->page = memory_pool_allocate( MEMORY_PAGE_SIZE );
+	  if( !mem[i]->page ) return 1;
+          mem[i]->writable = 1;
         }
-
+	
+        mem[i]->source = MEMORY_SOURCE_CARTRIDGE;
+	memcpy( mem[i]->page, dck->dck[num_block]->pages[i],
+		MEMORY_PAGE_SIZE );
         break;
 
       }
@@ -162,9 +177,10 @@ dck_read( const char *filename )
     num_block++;
   }
 
-  /* Make the menu item to eject the cartridge active */
-  ui_menu_activate( UI_MENU_ITEM_MEDIA_CARTRIDGE_EJECT, 1 );
+  dck_active = 1;
 
-  /* Don't deallocate the memory banks! */
-  return libspectrum_dck_free( dck, 1 );
+  /* Make the menu item to eject the cartridge active */
+  ui_menu_activate( UI_MENU_ITEM_MEDIA_CARTRIDGE_DOCK_EJECT, 1 );
+
+  return libspectrum_dck_free( dck, 0 );
 }
