@@ -1,7 +1,7 @@
 /* dck.c: dock snapshot (Warajevo .DCK) handling routines
    Copyright (c) 2003 Darren Salt, Fredrick Meunier
 
-   $Id: dck.h,v 1.1 2003/04/15 09:58:01 pak21 Exp $
+   $Id: dck.h,v 1.2 2004/05/25 10:05:06 fredm Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,8 +30,11 @@
 #ifndef FUSE_DCK_H
 #define FUSE_DCK_H
 
+/* Dock cart inserted? */
+extern int dck_active;
+
 int dck_insert( const char *filename );
 void dck_eject( void );
-int dck_read( const char *filename );
+int dck_reset( void );
 
 #endif
