@@ -1,7 +1,7 @@
 /* debugger_internals.h: The internals of Fuse's monitor/debugger
    Copyright (c) 2002-2003 Philip Kendall
 
-   $Id: debugger_internals.h,v 1.23 2003/12/29 14:39:41 pak21 Exp $
+   $Id: debugger_internals.h,v 1.25 2004/06/16 14:46:34 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -49,9 +49,26 @@ const char* debugger_register_text( int which );
 /* Utility functions called by the flex scanner */
 
 int debugger_command_input( char *buf, int *result, int max_size );
+int debugger_page_hash( const char *text );
 int yylex( void );
 void yyerror( const char *s );
 
+/* The semantic values of some tokens */
+
+typedef enum debugger_token {
+
+  /* Chosen to match up with Unicode values */
+  DEBUGGER_TOKEN_LOGICAL_AND = 0x2227,
+  DEBUGGER_TOKEN_LOGICAL_OR = 0x2228,
+
+  DEBUGGER_TOKEN_EQUAL_TO = 0x225f,
+  DEBUGGER_TOKEN_NOT_EQUAL_TO = 0x2260,
+
+  DEBUGGER_TOKEN_LESS_THAN_OR_EQUAL_TO = 0x2264,
+  DEBUGGER_TOKEN_GREATER_THAN_OR_EQUAL_TO = 0x2265,
+
+} debugger_token;
+
 /* Numeric expression stuff */
 
 debugger_expression*
