@@ -1,7 +1,7 @@
 /* disassemble.h: Fuse's disassembler
    Copyright (c) 2002-2003 Darren Salt, Philip Kendall
 
-   $Id: disassemble.c,v 1.9 2003/07/11 11:03:25 pak21 Exp $
+   $Id: disassemble.c,v 1.10 2003/11/14 11:46:47 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@
 
 #include "debugger.h"
 #include "fuse.h"
-#include "spectrum.h"
+#include "memory.h"
 
 /* Used to flag whether we're after a DD or FD prefix */
 enum hl_type { USE_HL, USE_IX, USE_IY };
