@@ -1,7 +1,7 @@
 /* expression.c: A numeric expression
    Copyright (c) 2003 Philip Kendall
 
-   $Id: expression.c,v 1.11 2003/12/29 14:39:41 pak21 Exp $
+   $Id: expression.c,v 1.13 2004/06/16 14:05:38 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -111,8 +111,6 @@ unaryop_precedence( int operation )
     ui_error( UI_ERROR_ERROR, "unknown unary operator %d", operation );
     fuse_abort();
   }
-
-  return 0;			/* Keep gcc happy */
 }
 
 static enum precedence_t
@@ -120,23 +118,27 @@ binaryop_precedence( int operation )
 {
   switch( operation ) {
 
-  case 0x2228:		    return PRECEDENCE_LOGICAL_OR;
-  case 0x2227:		    return PRECEDENCE_LOGICAL_AND;
+  case DEBUGGER_TOKEN_LOGICAL_OR: return PRECEDENCE_LOGICAL_OR;
+  case DEBUGGER_TOKEN_LOGICAL_AND: return PRECEDENCE_LOGICAL_AND;
   case    '|':		    return PRECEDENCE_BITWISE_OR;
   case    '^':		    return PRECEDENCE_BITWISE_XOR;
   case    '&':		    return PRECEDENCE_BITWISE_AND;
   case    '+': case    '-': return PRECEDENCE_ADDITION;
   case    '*': case    '/': return PRECEDENCE_MULTIPLICATION;
-  case 0x225f: case 0x2260: return PRECEDENCE_EQUALITY;
+
+  case DEBUGGER_TOKEN_EQUAL_TO:
+  case DEBUGGER_TOKEN_NOT_EQUAL_TO:
+    return PRECEDENCE_EQUALITY;
+
   case    '<': case    '>':
-  case 0x2264: case 0x2265: return PRECEDENCE_COMPARISION;
+  case DEBUGGER_TOKEN_LESS_THAN_OR_EQUAL_TO:
+  case DEBUGGER_TOKEN_GREATER_THAN_OR_EQUAL_TO:
+    return PRECEDENCE_COMPARISION;
 
   default:
     ui_error( UI_ERROR_ERROR, "unknown binary operator %d", operation );
     fuse_abort();
   }
-
-  return 0;			/* Keep gcc happy */
 }
 
 debugger_expression*
@@ -263,8 +265,6 @@ debugger_expression_evaluate( debugger_expression *exp )
 
   ui_error( UI_ERROR_ERROR, "unknown expression type %d", exp->type );
   fuse_abort();
-
-  return 0;			/* Keep gcc happy */
 }
 
 static libspectrum_dword
@@ -280,8 +280,6 @@ evaluate_unaryop( struct unaryop_type *unary )
 
   ui_error( UI_ERROR_ERROR, "unknown unary operator %d", unary->operation );
   fuse_abort();
-
-  return 0;			/* Keep gcc happy */
 }
 
 static libspectrum_dword
@@ -301,11 +299,13 @@ evaluate_binaryop( struct binaryop_type *binary )
   case '/': return debugger_expression_evaluate( binary->op1 ) /
 		   debugger_expression_evaluate( binary->op2 );
 
-  case 0x225f: return debugger_expression_evaluate( binary->op1 ) ==
-		      debugger_expression_evaluate( binary->op2 );
+  case DEBUGGER_TOKEN_EQUAL_TO:
+            return debugger_expression_evaluate( binary->op1 ) ==
+                   debugger_expression_evaluate( binary->op2 );
 
-  case 0x2260: return debugger_expression_evaluate( binary->op1 ) !=
-		      debugger_expression_evaluate( binary->op2 );
+  case DEBUGGER_TOKEN_NOT_EQUAL_TO:
+	    return debugger_expression_evaluate( binary->op1 ) !=
+		   debugger_expression_evaluate( binary->op2 );
 
   case '>': return debugger_expression_evaluate( binary->op1 ) >
 		   debugger_expression_evaluate( binary->op2 );
@@ -313,11 +313,13 @@ evaluate_binaryop( struct binaryop_type *binary )
   case '<': return debugger_expression_evaluate( binary->op1 ) <
 	           debugger_expression_evaluate( binary->op2 );
 
-  case 0x2264: return debugger_expression_evaluate( binary->op1 ) <=
-		      debugger_expression_evaluate( binary->op2 );
+  case DEBUGGER_TOKEN_LESS_THAN_OR_EQUAL_TO:
+	    return debugger_expression_evaluate( binary->op1 ) <=
+		   debugger_expression_evaluate( binary->op2 );
 
-  case 0x2265: return debugger_expression_evaluate( binary->op1 ) >=
-		      debugger_expression_evaluate( binary->op2 );
+  case DEBUGGER_TOKEN_GREATER_THAN_OR_EQUAL_TO:
+	    return debugger_expression_evaluate( binary->op1 ) >=
+		   debugger_expression_evaluate( binary->op2 );
 
   case '&': return debugger_expression_evaluate( binary->op1 ) &
 	           debugger_expression_evaluate( binary->op2 );
@@ -328,18 +330,18 @@ evaluate_binaryop( struct binaryop_type *binary )
   case '|': return debugger_expression_evaluate( binary->op1 ) |
 		   debugger_expression_evaluate( binary->op2 );
 
-  case 0x2227: return debugger_expression_evaluate( binary->op1 ) &&
+  case DEBUGGER_TOKEN_LOGICAL_AND:
+	    return debugger_expression_evaluate( binary->op1 ) &&
 		   debugger_expression_evaluate( binary->op2 );
 
-  case 0x2228: return debugger_expression_evaluate( binary->op1 ) ||
+  case DEBUGGER_TOKEN_LOGICAL_OR:
+	    return debugger_expression_evaluate( binary->op1 ) ||
 		   debugger_expression_evaluate( binary->op2 );
 
   }
 
   ui_error( UI_ERROR_ERROR, "unknown binary operator %d", binary->operation );
   fuse_abort();
-
-  return 0;			/* Keep gcc happy */
 }
 
 int
@@ -370,8 +372,6 @@ debugger_expression_deparse( char *buffer, size_t length,
 
   ui_error( UI_ERROR_ERROR, "unknown expression type %d", exp->type );
   fuse_abort();
-
-  return 0;			/* Keep gcc happy */
 }
   
 static int
@@ -444,17 +444,17 @@ deparse_binaryop( char *buffer, size_t length,
   case    '-': operation_string = "-";  break;
   case    '*': operation_string = "*";  break;
   case    '/': operation_string = "/";  break;
-  case 0x225f: operation_string = "=="; break;
-  case 0x2260: operation_string = "!="; break;
+  case DEBUGGER_TOKEN_EQUAL_TO: operation_string = "=="; break;
+  case DEBUGGER_TOKEN_NOT_EQUAL_TO: operation_string = "!="; break;
   case    '<': operation_string = "<";  break;
   case    '>': operation_string = ">";  break;
-  case 0x2264: operation_string = "<="; break;
-  case 0x2265: operation_string = ">="; break;
+  case DEBUGGER_TOKEN_LESS_THAN_OR_EQUAL_TO: operation_string = "<="; break;
+  case DEBUGGER_TOKEN_GREATER_THAN_OR_EQUAL_TO: operation_string = ">="; break;
   case    '&': operation_string = "&";  break;
   case    '^': operation_string = "^";  break;
   case    '|': operation_string = "|";  break;
-  case 0x2227: operation_string = "&&"; break;
-  case 0x2228: operation_string = "||"; break;
+  case DEBUGGER_TOKEN_LOGICAL_AND: operation_string = "&&"; break;
+  case DEBUGGER_TOKEN_LOGICAL_OR: operation_string = "||"; break;
 
   default:
     ui_error( UI_ERROR_ERROR, "unknown binary operation %d",
@@ -537,12 +537,16 @@ is_non_associative( int operation )
 
   /* None of the comparision operators are associative due to them
      returning truth values */
-  case 0x225f: case 0x2260: case '<': case '>': case 0x2264: case 0x2265:
+  case DEBUGGER_TOKEN_EQUAL_TO:
+  case DEBUGGER_TOKEN_NOT_EQUAL_TO:
+  case '<': case '>':
+  case DEBUGGER_TOKEN_LESS_THAN_OR_EQUAL_TO:
+  case DEBUGGER_TOKEN_GREATER_THAN_OR_EQUAL_TO:
     return 1;
 
   /* The logical operators are associative */
-  case 0x2227: return 0;
-  case 0x2228: return 0;
+  case DEBUGGER_TOKEN_LOGICAL_AND: return 0;
+  case DEBUGGER_TOKEN_LOGICAL_OR: return 0;
 
   /* The bitwise operators are also associative (consider them as
      vectorised logical operators) */
@@ -555,7 +559,5 @@ is_non_associative( int operation )
   /* Should never get here */
   ui_error( UI_ERROR_ERROR, "unknown binary operation %d", operation );
   fuse_abort();
-
-  return 0;			/* Keep gcc happy */
 }
 
