@@ -1,7 +1,7 @@
 /* event.c: Routines needed for dealing with the event list
    Copyright (c) 2000-2003 Philip Kendall
 
-   $Id: event.c,v 1.47 2003/12/29 14:39:41 pak21 Exp $
+   $Id: event.c,v 1.49 2004/03/26 16:35:36 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@
 #include "tape.h"
 #include "trdos.h"
 #include "ui/ui.h"
+#include "ui/uijoystick.h"
 #include "spectrum.h"
 #include "timer.h"
 #include "z80/z80.h"
@@ -133,9 +134,11 @@ int event_do_events(void)
       psg_frame();
       spectrum_frame();
       z80_interrupt();
+      ui_joystick_poll();
       timer_estimate_speed();
       debugger_add_time_events();
       ui_event();
+      ui_error_frame();
       break;
 
     case EVENT_TYPE_INTERRUPT:
