@@ -1,7 +1,7 @@
 /* fuse.c: The Free Unix Spectrum Emulator
    Copyright (c) 1999-2004 Philip Kendall
 
-   $Id: fuse.c,v 1.107.2.2 2004/02/11 21:50:03 pak21 Exp $
+   $Id: fuse.c,v 1.120 2004/05/31 17:38:19 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -52,21 +52,25 @@
 #include "joystick.h"
 #include "keyboard.h"
 #include "machine.h"
+#include "memory.h"
 #include "pokefinder/pokefinder.h"
 #include "printer.h"
 #include "psg.h"
 #include "rzx.h"
 #include "settings.h"
+#include "simpleide.h"
 #include "snapshot.h"
 #include "sound.h"
+#include "sound/lowlevel.h"
 #include "spectrum.h"
-#include "specplus3.h"
 #include "tape.h"
 #include "timer.h"
 #include "trdos.h"
 #include "ui/ui.h"
 #include "ui/scaler/scaler.h"
 #include "utils.h"
+#include "zxatasp.h"
+#include "zxcf.h"
 
 #ifdef USE_WIDGET
 #include "widget/widget.h"
@@ -104,11 +108,16 @@ typedef struct start_files_t {
   const char *disk_plus3;
   const char *disk_trdos;
   const char *dock;
+  const char *if2;
   const char *playback;
   const char *recording;
   const char *snapshot;
   const char *tape;
 
+  const char *simpleide_master, *simpleide_slave;
+  const char *zxatasp_master, *zxatasp_slave;
+  const char *zxcf;
+
 } start_files_t;
 
 static int fuse_init(int argc, char **argv);
@@ -213,12 +222,17 @@ static int fuse_init(int argc, char **argv)
   if( !geteuid() ) { setuid( getuid() ); }
 #endif				/* #ifdef HAVE_GETEUID */
 
+  if( memory_init() ) return 1;
+
   if( debugger_init() ) return 1;
 
   if( printer_init() ) return 1;
   if( rzx_init() ) return 1;
   if( psg_init() ) return 1;
   if( trdos_init() ) return 1;
+  if( simpleide_init() ) return 1;
+  if( zxatasp_init() ) return 1;
+  if( zxcf_init() ) return 1;
 
   error = pokefinder_clear(); if( error ) return error;
 
@@ -395,6 +409,7 @@ static void fuse_show_help( void )
    "--loading-sound        Emulate the sound of tapes loading.\n"
    "--separation           Use ACB stereo for the AY-3-8912 sound chip.\n"
    "--sound                Produce sound.\n"
+   "--sound-force-8bit     Generate 8-bit sound even if 16-bit is available.\n"
    "--slt                  Turn SLT traps on.\n"
    "--traps                Turn tape traps on.\n\n"
    "Other options:\n\n"
@@ -493,11 +508,20 @@ setup_start_files( start_files_t *start_files )
   start_files->disk_plus3 = settings_current.plus3disk_file;
   start_files->disk_trdos = settings_current.trdosdisk_file;
   start_files->dock = settings_current.dck_file;
+  start_files->if2 = settings_current.if2_file;
   start_files->playback = settings_current.playback_file;
   start_files->recording = settings_current.record_file;
   start_files->snapshot = settings_current.snapshot;
   start_files->tape = settings_current.tape_file;
 
+  start_files->simpleide_master = settings_current.simpleide_master_file;
+  start_files->simpleide_slave  = settings_current.simpleide_slave_file;
+
+  start_files->zxatasp_master = settings_current.zxatasp_master_file;
+  start_files->zxatasp_slave  = settings_current.zxatasp_slave_file;
+
+  start_files->zxcf = settings_current.zxcf_pri_file;
+
   return 0;
 }
 
@@ -529,6 +553,23 @@ parse_nonoption_args( int argc, char **argv, int first_arg,
     case LIBSPECTRUM_CLASS_CARTRIDGE_TIMEX:
       start_files->dock = filename; break;
 
+    case LIBSPECTRUM_CLASS_CARTRIDGE_IF2:
+      start_files->if2 = filename; break;
+
+    case LIBSPECTRUM_CLASS_HARDDISK:
+      if( settings_current.zxcf_active ) {
+	start_files->zxcf = filename;
+      } else if( settings_current.zxatasp_active ) {
+	start_files->zxatasp_master = filename;
+      } else if( settings_current.simpleide_active ) {
+	start_files->simpleide_master = filename;
+      } else {
+	/* No IDE interface active, so activate the ZXCF */
+	settings_current.zxcf_active = 1;
+	start_files->zxcf = filename;
+      }
+      break;
+
     case LIBSPECTRUM_CLASS_DISK_PLUS3:
       start_files->disk_plus3 = filename; break;
 
@@ -545,7 +586,7 @@ parse_nonoption_args( int argc, char **argv, int first_arg,
       start_files->tape = filename; break;
 
     case LIBSPECTRUM_CLASS_UNKNOWN:
-      ui_error( UI_ERROR_INFO, "couldn't identify '%s'; ignoring it",
+      ui_error( UI_ERROR_WARNING, "couldn't identify '%s'; ignoring it",
 		filename );
       break;
 
@@ -568,7 +609,7 @@ do_start_files( start_files_t *start_files )
   /* Can't do both input recording and playback */
   if( start_files->playback && start_files->recording ) {
     ui_error(
-      UI_ERROR_INFO,
+      UI_ERROR_WARNING,
       "can't do both input playback and recording; recording disabled"
     );
     start_files->recording = NULL;
@@ -577,7 +618,7 @@ do_start_files( start_files_t *start_files )
   /* Can't use both +3 and TR-DOS disks simultaneously */
   if( start_files->disk_plus3 && start_files->disk_trdos ) {
     ui_error(
-      UI_ERROR_INFO,
+      UI_ERROR_WARNING,
       "can't use +3 and TR-DOS disks simultaneously; +3 disk ignored"
     );
     start_files->disk_plus3 = NULL;
@@ -587,12 +628,22 @@ do_start_files( start_files_t *start_files )
   if( ( start_files->disk_plus3 || start_files->disk_trdos ) &&
       start_files->dock                                         ) {
     ui_error(
-      UI_ERROR_INFO,
+      UI_ERROR_WARNING,
       "can't use disks and the dock simultaneously; dock cartridge ignored"
     );
     start_files->dock = NULL;
   }
 
+  /* Can't use disks and the Interface II simultaneously */
+  if( ( start_files->disk_plus3 || start_files->disk_trdos ) &&
+      start_files->if2                                          ) {
+    ui_error(
+      UI_ERROR_WARNING,
+      "can't use disks and the Interface II simultaneously; cartridge ignored"
+    );
+    start_files->if2 = NULL;
+  }
+
   /* If a snapshot has been specified, don't autoload tape, disks etc */
   autoload = start_files->snapshot ? 0 : settings_current.auto_load;
 
@@ -617,6 +668,11 @@ do_start_files( start_files_t *start_files )
     if( error ) return error;
   }
 
+  if( start_files->if2 ) {
+    error = utils_open_file( start_files->if2, autoload, NULL );
+    if( error ) return error;
+  }
+
   if( start_files->snapshot ) {
     error = utils_open_file( start_files->snapshot, autoload, NULL );
     if( error ) return error;
@@ -627,6 +683,38 @@ do_start_files( start_files_t *start_files )
     if( error ) return error;
   }
 
+  /* IDE hard disk images */
+
+  if( start_files->simpleide_master ) {
+    error = simpleide_insert( start_files->simpleide_master,
+			      LIBSPECTRUM_IDE_MASTER );
+    simpleide_reset();
+    if( error ) return error;
+  }
+
+  if( start_files->simpleide_slave ) {
+    error = simpleide_insert( start_files->simpleide_slave,
+			      LIBSPECTRUM_IDE_SLAVE );
+    simpleide_reset();
+    if( error ) return error;
+  }
+
+  if( start_files->zxatasp_master ) {
+    error = zxatasp_insert( start_files->zxatasp_master,
+			    LIBSPECTRUM_IDE_MASTER );
+    if( error ) return error;
+  }
+
+  if( start_files->zxatasp_slave ) {
+    error = zxatasp_insert( start_files->zxatasp_slave,
+			    LIBSPECTRUM_IDE_SLAVE );
+    if( error ) return error;
+  }
+
+  if( start_files->zxcf ) {
+    error = zxcf_insert( start_files->zxcf ); if( error ) return error;
+  }
+
   /* Input recordings */
 
   if( start_files->playback ) {
@@ -653,6 +741,9 @@ static int fuse_end(void)
   psg_end();
   rzx_end();
   debugger_end();
+  simpleide_end();
+  zxatasp_end();
+  zxcf_end();
 
   machine_end();
 
@@ -679,7 +770,8 @@ static int fuse_end(void)
 }
 
 /* Emergency shutdown */
-int fuse_abort( void )
+void
+fuse_abort( void )
 {
   fuse_end();
   abort();
