@@ -1,7 +1,7 @@
 /* keyboard.h: Routines for dealing with the Spectrum's keyboard
    Copyright (c) 1999-2003 Philip Kendall
 
-   $Id: keyboard.h,v 1.14 2003/07/11 11:03:25 pak21 Exp $
+   $Id: keyboard.h,v 1.18 2004/04/16 18:16:39 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,6 +29,10 @@
 
 #include <libspectrum.h>
 
+#ifndef FUSE_INPUT_H
+#include "input.h"
+#endif			/* #ifndef FUSE_INPUT_H */
+
 extern libspectrum_byte keyboard_default_value;
 extern libspectrum_byte keyboard_return_values[8];
 
@@ -82,18 +86,10 @@ typedef enum keyboard_key_name {
   KEYBOARD_Caps,
   KEYBOARD_Symbol,
 
-  /* Below here, keys don't actually correspond to Spectrum keys, but
-     are used by the widget code */
+  /* Used by the configuration code to signify that a real joystick fire
+     button should map to the emulated joystick fire button */
 
-  KEYBOARD_PageUp = 0x200,
-  KEYBOARD_PageDown,
-  KEYBOARD_Home,
-  KEYBOARD_End,
-
-  /* Fake keypress used on window resize to cause widgets to redraw
-     themselves */
-
-  KEYBOARD_Resize = 0x400,
+  KEYBOARD_JOYSTICK_FIRE = 0x1000,
 
 } keyboard_key_name;
 
@@ -103,4 +99,32 @@ void keyboard_press(keyboard_key_name key);
 void keyboard_release(keyboard_key_name key);
 int keyboard_release_all( void );
 
+/* Which Spectrum keys should be emulated as pressed when each input
+   layer key is pressed */
+
+typedef struct keyboard_spectrum_keys_t {
+
+  keyboard_key_name key1,key2;
+
+} keyboard_spectrum_keys_t;
+
+const keyboard_spectrum_keys_t* keyboard_get_spectrum_keys( input_key keysym );
+
+/* The mapping from UI layer keysyms to Fuse input layer keysyms */
+
+typedef struct keysyms_map_t {
+
+  /* FIXME: this should really be the UI specific type used for keysyms */
+  libspectrum_dword ui;
+
+  input_key fuse;
+
+} keysyms_map_t;
+
+extern keysyms_map_t keysyms_map[];
+
+input_key keysyms_remap( libspectrum_dword ui_keysym );
+
+const char* keyboard_key_text( keyboard_key_name key );
+
 #endif			/* #ifndef FUSE_KEYBOARD_H */
