@@ -1,7 +1,7 @@
 /* machine.c: Routines for handling the various machine types
-   Copyright (c) 1999-2003 Philip Kendall
+   Copyright (c) 1999-2004 Philip Kendall
 
-   $Id: machine.c,v 1.82 2004/01/10 18:47:17 pak21 Exp $
+   $Id: machine.c,v 1.101 2004/06/09 10:54:51 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,26 +31,27 @@
 
 #include "event.h"
 #include "fuse.h"
+#include "if2.h"
 #include "machine.h"
-#include "pentagon.h"
+#include "machines/machines.h"
+#include "machines/scorpion.h"
+#include "machines/spec128.h"
+#include "machines/spec48.h"
+#include "machines/specplus3.h"
 #include "printer.h"
 #include "scld.h"
 #include "settings.h"
+#include "simpleide.h"
 #include "snapshot.h"
 #include "sound.h"
-#include "spec16.h"
-#include "spec48.h"
-#include "spec128.h"
-#include "specplus2.h"
-#include "specplus2a.h"
-#include "specplus3.h"
-#include "tc2048.h"
-#include "tc2068.h"
 #include "tape.h"
 #include "ui/ui.h"
 #include "ui/uidisplay.h"
+#include "ula.h"
 #include "utils.h"
 #include "z80/z80.h"
+#include "zxatasp.h"
+#include "zxcf.h"
 
 fuse_machine_info **machine_types = NULL; /* Array of available machines */
 int machine_count = 0;
@@ -82,6 +83,11 @@ int machine_init_machines( void )
      emulations are identical */
   error = machine_add_machine( specplus3_init );
   if (error ) return error;
+
+  /* FIXME: what should we do about the +3e if we don't have lib765? */
+  error = machine_add_machine( specplus3e_init );
+  if( error ) return error;
+
 #endif				/* #ifdef HAVE_765_H */
 
   error = machine_add_machine( tc2048_init );
@@ -90,33 +96,41 @@ int machine_init_machines( void )
   if( error ) return error;
   error = machine_add_machine( pentagon_init );
   if (error ) return error;
+  error = machine_add_machine( scorpion_init );
+  if ( error ) return error;
+  error = machine_add_machine( spec_se_init );
+  if ( error ) return error;
 
   return 0;
 }
 
 static int machine_add_machine( int (*init_function)( fuse_machine_info *machine ) )
 {
+  fuse_machine_info *machine;
   int error;
 
   machine_count++;
 
-  machine_types = 
-    (fuse_machine_info**)realloc( machine_types, 
-			     machine_count * sizeof( fuse_machine_info* ) );
+  machine_types = realloc( machine_types,
+			   machine_count * sizeof( fuse_machine_info* ) );
   if( machine_types == NULL ) {
     ui_error( UI_ERROR_ERROR, "out of memory at %s:%d", __FILE__, __LINE__ );
     return 1;
   }
 
-  machine_types[ machine_count - 1 ] =
-    (fuse_machine_info*)malloc( sizeof( fuse_machine_info ) );
-  if( machine_types[ machine_count - 1 ] == NULL ) {
+  machine_types[ machine_count - 1 ] = malloc( sizeof( fuse_machine_info ) );
+  if( !machine_types[ machine_count - 1 ] ) {
     ui_error( UI_ERROR_ERROR, "out of memory at %s:%d", __FILE__, __LINE__ );
     return 1;
   }
 
-  error = init_function( machine_types[ machine_count - 1 ] );
-  if( error ) return error;
+  machine = machine_types[ machine_count - 1 ];
+
+  error = init_function( machine ); if( error ) return error;
+
+  error = machine_set_timings( machine ); if( error ) return error;
+
+  machine->capabilities = libspectrum_machine_capabilities( machine->machine );
 
   return 0;
 }
@@ -201,9 +215,6 @@ machine_select_machine( fuse_machine_info *machine )
     return 1;
   if( event_add( machine->line_times[0], EVENT_TYPE_LINE) ) return 1;
 
-  read_screen_memory = machine->ram.read_screen;
-  contend_port = machine->ram.contend_port;
-  
   if( uidisplay_end() ) return 1;
 
   capabilities = libspectrum_machine_capabilities( machine->machine );
@@ -233,21 +244,22 @@ machine_select_machine( fuse_machine_info *machine )
   }
 
   if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_TIMEX_DOCK ) {
-    ui_menu_activate( UI_MENU_ITEM_MEDIA_CARTRIDGE, 1 );
-    ui_menu_activate( UI_MENU_ITEM_MEDIA_CARTRIDGE_EJECT, 0 );
+    ui_menu_activate( UI_MENU_ITEM_MEDIA_CARTRIDGE_DOCK, 1 );
+    ui_menu_activate( UI_MENU_ITEM_MEDIA_CARTRIDGE_DOCK_EJECT, 0 );
   } else {
-    ui_menu_activate( UI_MENU_ITEM_MEDIA_CARTRIDGE, 0 );
+    ui_menu_activate( UI_MENU_ITEM_MEDIA_CARTRIDGE_DOCK, 0 );
   };
 
   return 0;
 }
 
 int
-machine_load_rom( libspectrum_byte **data, char *filename,
-		  size_t expected_length )
+machine_load_rom_bank( memory_page* bank_map, size_t which, int page_num,
+		       const char *filename, size_t expected_length )
 {
   int fd, error;
   utils_file rom;
+  size_t i, offset;
 
   fd = utils_find_auxiliary_file( filename, UTILS_AUXILIARY_ROM );
   if( fd == -1 ) {
@@ -263,23 +275,41 @@ machine_load_rom( libspectrum_byte **data, char *filename,
 	      "ROM '%s' is %ld bytes long; expected %ld bytes",
 	      filename, (unsigned long)rom.length,
 	      (unsigned long)expected_length );
+    utils_close_file( &rom );
     return 1;
   }
 
-  /* Take a copy of the ROM in case we want to write to it later */
-  *data = malloc( rom.length * sizeof( libspectrum_byte ) );
-  if( !(*data) ) {
-    ui_error( UI_ERROR_ERROR, "out of memory at %s:%d", __FILE__, __LINE__ );
+  bank_map[ which ].offset = 0;
+  bank_map[ which ].page_num = page_num;
+  bank_map[ which ].page = memory_pool_allocate( rom.length );
+  if( !bank_map[ which ].page ) {
+    utils_close_file( &rom );
     return 1;
   }
 
-  memcpy( *data, rom.buffer, rom.length );
+  memcpy( bank_map[ which ].page, rom.buffer, rom.length );
+
+  for( i = 1, offset = MEMORY_PAGE_SIZE;
+       offset < expected_length;
+       i++, offset += MEMORY_PAGE_SIZE   ) {
+    bank_map[ which + i ].offset = offset;
+    bank_map[ which + i ].page_num = page_num;
+    bank_map[ which + i ].page = bank_map[ which ].page + offset;
+  }
 
   if( utils_close_file( &rom ) ) return 1;
 
   return 0;
 }
 
+int
+machine_load_rom( size_t which, int page_num, const char *filename,
+		  size_t expected_length )
+{
+  return machine_load_rom_bank( memory_map_rom, which, page_num, filename,
+				expected_length );
+}
+
 int
 machine_reset( void )
 {
@@ -293,36 +323,28 @@ machine_reset( void )
      sound_frame() or sound_ay_write() */
   sound_ay_reset();
 
-  snapshot_flush_slt();
   printer_zxp_reset();
   scld_reset();
   tape_stop();
+  simpleide_reset();
+  zxatasp_reset();
+  zxcf_reset();
 
-  /* Load in the ROMs: remove any ROMs we've got in memory at the moment */
-  for( i = 0; i < spectrum_rom_count; i++ ) {
-    if( ROM[i] ) { free( ROM[i] ); ROM[i] = 0; }
-  }
-    
-  /* Make sure we have enough space for the new ROMs */
-  if( spectrum_rom_count < machine_current->rom_count ) {
+  memory_pool_free();
 
-    libspectrum_byte **new_ROM =
-      realloc( ROM, machine_current->rom_count * sizeof( libspectrum_byte* ) );
-    if( !new_ROM ) {
-      ui_error( UI_ERROR_ERROR, "out of memory at %s:%d", __FILE__, __LINE__ );
-      return 1;
-    }
-
-    ROM = new_ROM;
-  }
-  spectrum_rom_count = machine_current->rom_count;
+  machine_current->ram.romcs = 0;
 
   /* Do the machine-specific bits, including loading the ROMs */
   error = machine_current->reset(); if( error ) return error;
 
+  error = machine_current->memory_map(); if( error ) return error;
+
   /* Set up the contention array */
   for( i = 0; i < machine_current->timings.tstates_per_frame; i++ )
-    spectrum_contention[ i ] = machine_current->ram.contend_delay( i );
+    ula_contention[ i ] = machine_current->ram.contend_delay( i );
+
+  /* Check for an Interface II ROM */
+  error = if2_reset(); if( error ) return error;
 
   return 0;
 }
@@ -359,12 +381,16 @@ machine_set_timings( fuse_machine_info *machine )
      appropriate number of left border cycles; the difference between this
      and the actually displayed width is accounted for in
      display.c:display_border_column()
+
+     The addition of 3 at the end accounts for the fact that the ULA
+     receives the new data at T2 of the IO cycle of an OUT, but the
+     Z80 core doesn't actually output it until after T4.
   */
 
   machine->line_times[0]=
     libspectrum_timings_top_left_pixel( machine->machine ) -
     24 * machine->timings.tstates_per_line -
-    machine->timings.left_border;
+    machine->timings.left_border + 3;
 
   for( y=1; y<DISPLAY_SCREEN_HEIGHT+1; y++ ) {
     machine->line_times[y] = machine->line_times[y-1] + 
@@ -374,26 +400,12 @@ machine_set_timings( fuse_machine_info *machine )
   return 0;
 }
 
-int machine_allocate_roms( fuse_machine_info *machine, size_t count )
-{
-  machine->rom_count = count;
-
-  machine->rom_length = malloc( count * sizeof(size_t) );
-  if( machine->rom_length == NULL ) {
-    ui_error( UI_ERROR_ERROR, "out of memory at %s:%d", __FILE__, __LINE__ );
-    return 1;
-  }
-
-  return 0;
-}
-
 int machine_end( void )
 {
   int i;
 
   for( i=0; i<machine_count; i++ ) {
     if( machine_types[i]->shutdown ) machine_types[i]->shutdown();
-    free( machine_types[i]->rom_length );
     free( machine_types[i] );
   }
 
