@@ -1,7 +1,7 @@
 /* machine.h: Routines for handling the various machine types
-   Copyright (c) 1999-2003 Philip Kendall
+   Copyright (c) 1999-2004 Philip Kendall
 
-   $Id: machine.h,v 1.40 2003/12/02 14:56:32 pak21 Exp $
+   $Id: machine.h,v 1.46 2004/05/14 12:35:01 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,23 +27,14 @@
 #ifndef FUSE_MACHINE_H
 #define FUSE_MACHINE_H
 
-#ifndef _STDLIB_H
 #include <stdlib.h>
-#endif			/* #ifndef _STDLIB_H */
 
 #include <libspectrum.h>
 
-#ifndef FUSE_AY_H
 #include "ay.h"
-#endif			/* #ifndef FUSE_AY_H */
-
-#ifndef FUSE_DISPLAY_H
 #include "display.h"
-#endif			/* #ifndef FUSE_DISPLAY_H */
-
-#ifndef FUSE_SPECTRUM_H
+#include "memory.h"
 #include "spectrum.h"
-#endif			/* #ifndef FUSE_SPECTRUM_H */
 
 typedef libspectrum_byte (*spectrum_unattached_port_fn)( void );
 
@@ -67,6 +58,7 @@ typedef struct fuse_machine_info {
 
   libspectrum_machine machine; /* which machine type is this? */
   const char *id;	/* Used to select from command line */
+  int capabilities;	/* Capabilities of this machine */
 
   int (*reset)(void);	/* Reset function */
 
@@ -79,10 +71,6 @@ typedef struct fuse_machine_info {
   spectrum_raminfo ram; /* How do we access memory, and what's currently
 			   paged in */
 
-  size_t rom_count;	/* How many ROMs does this machine use? */
-  size_t *rom_length;	/* And how long is each ROM? */
-
-  spectrum_port_info *peripherals; /* Which peripherals do we have? */
   spectrum_unattached_port_fn unattached_port; /* What to return if we read
 						  from a port which isn't
 						  attached to anything */
@@ -91,6 +79,8 @@ typedef struct fuse_machine_info {
 
   int (*shutdown)( void );
 
+  int (*memory_map)( void );
+
 } fuse_machine_info;
 
 extern fuse_machine_info **machine_types;	/* All available machines */
@@ -106,8 +96,9 @@ const char* machine_get_id( libspectrum_machine type );
 
 int machine_set_timings( fuse_machine_info *machine );
 
-int machine_allocate_roms( fuse_machine_info *machine, size_t count );
-int machine_load_rom( libspectrum_byte **data, char *filename,
+int machine_load_rom_bank( memory_page* bank_map, size_t which, int page_num,
+			   const char *filename, size_t expected_length );
+int machine_load_rom( size_t which, int page_num, const char *filename,
 		      size_t expected_length );
 
 int machine_reset( void );
