@@ -1,7 +1,7 @@
 /* pokefinder.h: help with finding pokes
-   Copyright (c) 2003 Philip Kendall
+   Copyright (c) 2003-2004 Philip Kendall
 
-   $Id: pokefinder.h,v 1.1 2003/10/23 21:10:43 pak21 Exp $
+   $Id: pokefinder.h,v 1.2 2004/03/09 16:13:43 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,10 +29,12 @@
 
 #include <libspectrum.h>
 
-extern libspectrum_byte pokefinder_possible[8][0x4000];
+extern int pokefinder_possible[8][0x4000];
 extern size_t pokefinder_count;
 
 int pokefinder_clear( void );
 int pokefinder_search( libspectrum_byte value );
+int pokefinder_incremented( void );
+int pokefinder_decremented( void );
 
 #endif				/* #ifndef FUSE_POKEFINDER_H */
