@@ -1,7 +1,7 @@
 /* printer.c: Printer support
-   Copyright (c) 2001-2003 Ian Collier, Russell Marks, Philip Kendall
+   Copyright (c) 2001-2004 Ian Collier, Russell Marks, Philip Kendall
 
-   $Id: printer.c,v 1.17 2003/11/14 11:46:46 pak21 Exp $
+   $Id: printer.c,v 1.19 2004/04/19 21:47:15 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -373,13 +373,16 @@ frames++;
  * very little brain. :-) Or at least, I don't grok it that well.
  * It works wonderfully though.
  */
-libspectrum_byte printer_zxp_read(libspectrum_word port GCC_UNUSED)
+libspectrum_byte printer_zxp_read(libspectrum_word port GCC_UNUSED,
+				  int *attached)
 {
 if(!settings_current.printer)
   return(0xff);
 if(!printer_graphics_enabled)
   return(0xff);
 
+*attached=1;
+
 if(!zxpspeed)
   return 0x3e;
 else
@@ -526,7 +529,8 @@ void printer_serial_write(libspectrum_byte b)
 {
 static int reading=0,bits_to_get=0,ser_byte=0;
 int high=(b&8);
-
+if(!settings_current.printer)
+  return;
 if(!reading)
   {
   if(!high)
@@ -574,6 +578,8 @@ static libspectrum_dword last_tstates=0;
 static unsigned char last_data=0;
 libspectrum_dword diff;
 
+if(!settings_current.printer)
+  return;
 if((old_on && !on) || (!old_on && on))
   {
   /* got an edge */
@@ -613,10 +619,14 @@ old_on=on;
 }
 
 
-libspectrum_byte printer_parallel_read(libspectrum_word port GCC_UNUSED)
+libspectrum_byte printer_parallel_read(libspectrum_word port GCC_UNUSED,
+				       int *attached)
 {
 if(!settings_current.printer)
   return(0xff);
+
+*attached = 1;
+
 /* bit 0 = busy. other bits high? */
 
 return(0xfe);	/* never busy */
