@@ -1,7 +1,7 @@
 /* rzx.h: .rzx files
    Copyright (c) 2002-2003 Philip Kendall
 
-   $Id: rzx.h,v 1.18 2003/08/17 20:16:43 pak21 Exp $
+   $Id: rzx.h,v 1.19 2004/04/15 19:21:47 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -63,7 +63,7 @@ int rzx_init( void );
 int rzx_start_recording( const char *filename, int embed_snapshot );
 int rzx_stop_recording( void );
 
-int rzx_start_playback( const char *filename, int (*load_snap)(void) );
+int rzx_start_playback( const char *filename );
 int
 rzx_start_playback_from_buffer( const unsigned char *buffer, size_t length );
 
