@@ -1,7 +1,7 @@
 /* screenshot.c: Routines for handling .png and .scr screenshots
    Copyright (c) 2002-2003 Philip Kendall, Fredrick Meunier
 
-   $Id: screenshot.c,v 1.19.2.3 2004/02/09 12:25:45 pak21 Exp $
+   $Id: screenshot.c,v 1.21 2004/07/04 18:48:36 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -186,7 +186,7 @@ get_rgb32_data( libspectrum_byte *rgb32_data, size_t stride,
 {
   size_t i, x, y;
 
-				      /*  R    G    B */
+  static const			      /*  R    G    B */
   libspectrum_byte palette[16][3] = { {   0,   0,   0 },
 				      {   0,   0, 192 },
 				      { 192,   0,   0 },
