@@ -1,5 +1,7 @@
 /* sound.c: Sound support
-   Copyright (c) 2000-2003 Russell Marks, Matan Ziv-Av, Philip Kendall
+   Copyright (c) 2000-2004 Russell Marks, Matan Ziv-Av, Philip Kendall
+
+   $Id: sound.c,v 1.44 2004/07/04 18:48:37 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -41,14 +43,11 @@
 #include "machine.h"
 #include "settings.h"
 #include "sound.h"
+#include "sound/lowlevel.h"
 #include "ui/ui.h"
 
 /* Do we have any of our sound devices available? */
 
-#if defined(HAVE_SYS_SOUNDCARD_H) || defined(HAVE_SYS_AUDIOIO_H) || defined(UI_SDL) || defined(HAVE_DSOUND_H)
-#define HAVE_SOUND
-#endif
-
 /* configuration */
 int sound_enabled=0;		/* Are we currently using the sound card;
 				   cf fuse.c:fuse_sound_in_use */
@@ -65,10 +64,11 @@ static int sound_stereo_beeper=0;   /* and settings_current.stereo_beeper */
 
 /* assume all three tone channels together match the beeper volume (ish).
  * Must be <=127 for all channels; 50+2+(24*3) = 124.
+ * (Now scaled up for 16-bit.)
  */
-#define AMPL_BEEPER		50
-#define AMPL_TAPE		2
-#define AMPL_AY_TONE		24	/* three of these */
+#define AMPL_BEEPER		(50*256)
+#define AMPL_TAPE		(2*256)
+#define AMPL_AY_TONE		(24*256)	/* three of these */
 
 /* max. number of sub-frame AY port writes allowed;
  * given the number of port writes theoretically possible in a
@@ -80,9 +80,9 @@ static int sound_framesiz;
 
 static int sound_channels;
 
-static unsigned char ay_tone_levels[16];
+static unsigned int ay_tone_levels[16];
 
-static unsigned char *sound_buf,*tape_buf;
+static libspectrum_signed_word *sound_buf,*tape_buf;
 
 /* beeper stuff */
 static int sound_oldpos[2],sound_fillpos[2];
@@ -129,7 +129,7 @@ static void sound_ay_init(void)
  * Matthew Westcott, adjusted as I described in a followup to his post,
  * then scaled to 0..0xffff.
  */
-static int levels[16]=
+static const int levels[16]=
   {
   0x0000, 0x0385, 0x053D, 0x0770,
   0x0AD7, 0x0FD5, 0x15B0, 0x230C,
@@ -199,8 +199,9 @@ sound_enabled=sound_enabled_ever=1;
 sound_channels=(sound_stereo?2:1);
 sound_framesiz=sound_freq/50;
 
-if((sound_buf=malloc(sound_framesiz*sound_channels))==NULL ||
-   (tape_buf=malloc(sound_framesiz))==NULL)
+if((sound_buf=malloc(sizeof(libspectrum_signed_word)*
+                     sound_framesiz*sound_channels))==NULL ||
+   (tape_buf=malloc(sizeof(libspectrum_signed_word)*sound_framesiz))==NULL)
   {
   if(sound_buf)
     {
@@ -228,7 +229,7 @@ if(first_init)
   first_init=0;
 
   for(f=0;f<2;f++)
-    sound_oldval[f]=sound_oldval_orig[f]=128;
+    sound_oldval[f]=sound_oldval_orig[f]=0;
   }
 
 if(sound_stereo_beeper)
@@ -272,20 +273,19 @@ if(sound_enabled)
 
 
 /* write sample to buffer as pseudo-stereo */
-static void sound_write_buf_pstereo(unsigned char *out,int c)
+static void sound_write_buf_pstereo(libspectrum_signed_word *out,int c)
 {
-int bl=(c-128-pstereobuf[pstereopos])/2;
-int br=(c-128+pstereobuf[pstereopos])/2;
+int bl=(c-pstereobuf[pstereopos])/2;
+int br=(c+pstereobuf[pstereopos])/2;
 
 if(bl<-AMPL_BEEPER) bl=-AMPL_BEEPER;
 if(br<-AMPL_BEEPER) br=-AMPL_BEEPER;
 if(bl> AMPL_BEEPER) bl= AMPL_BEEPER;
 if(br> AMPL_BEEPER) br= AMPL_BEEPER;
 
-bl+=128; br+=128;
-*out=(unsigned char)bl; out[1]=(unsigned char)br;
+*out=bl; out[1]=br;
 
-pstereobuf[pstereopos]=c-128;
+pstereobuf[pstereopos]=c;
 pstereopos++;
 if(pstereopos>=pstereobufsiz)
   pstereopos=0;
@@ -368,7 +368,7 @@ static int env_first=1,env_rev=0,env_counter=15;
 int tone_level[3];
 int mixer,envshape;
 int f,g,level,count;
-unsigned char *ptr;
+libspectrum_signed_word *ptr;
 struct ay_change_tag *change_ptr=ay_change;
 int changes_left=ay_change_count;
 int reg,r;
@@ -377,7 +377,7 @@ int chan1,chan2,chan3;
 unsigned int tone_count,noise_count;
 
 /* If no AY chip, don't produce any AY sound (!) */
-if(!machine_current->ay.present) return;
+if(!machine_current->capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_AY) return;
 
 /* convert change times to sample offsets */
 for(f=0;f<ay_change_count;f++)
@@ -670,7 +670,7 @@ ay_tone_subcycles=ay_env_subcycles=0;
 
 void sound_frame(void)
 {
-unsigned char *ptr,*tptr;
+libspectrum_signed_word *ptr,*tptr;
 int f,bchan;
 int ampl=AMPL_BEEPER;
 
@@ -692,9 +692,9 @@ ptr=sound_buf;
 tptr=tape_buf;
 for(f=0;f<sound_framesiz;f++,tptr++)
   {
-  (*ptr++)+=*tptr-128;
+  (*ptr++)+=*tptr;
   if(sound_stereo)
-    (*ptr++)+=*tptr-128;
+    (*ptr++)+=*tptr;
   }
 
 /* overlay AY sound */
@@ -714,7 +714,7 @@ ay_change_count=0;
  */
 void sound_beeper(int is_tape,int on)
 {
-unsigned char *ptr;
+libspectrum_signed_word *ptr;
 int newpos,subpos;
 int val,subval;
 int f;
@@ -724,7 +724,7 @@ int vol=ampl*2;
 
 if(!sound_enabled) return;
 
-val=(on?128-ampl:128+ampl);
+val=(on?-ampl:ampl);
 
 if(val==sound_oldval_orig[bchan]) return;
 
@@ -732,9 +732,8 @@ if(val==sound_oldval_orig[bchan]) return;
  * whenever cycles_per_frame were changed (i.e. when machine type changed).
  */
 newpos=(tstates*sound_framesiz)/machine_current->timings.tstates_per_frame;
-/* the >>1s are to avoid overflow when int is 32-bit */
-subpos=((tstates>>1)*sound_framesiz*vol)/
-       (machine_current->timings.tstates_per_frame>>1)-vol*newpos;
+subpos=(((libspectrum_signed_qword)tstates)*sound_framesiz*vol)/
+       (machine_current->timings.tstates_per_frame)-vol*newpos;
 
 /* if we already wrote here, adjust the level.
  */
@@ -752,7 +751,7 @@ if(newpos==sound_oldpos[bchan])
 else
   beeper_last_subpos[bchan]=(on?vol-subpos:subpos);
 
-subval=128+ampl-beeper_last_subpos[bchan];
+subval=ampl-beeper_last_subpos[bchan];
 
 if(newpos>=0)
   {
@@ -782,25 +781,3 @@ sound_oldpos[bchan]=newpos;
 sound_fillpos[bchan]=newpos+1;
 sound_oldval[bchan]=sound_oldval_orig[bchan]=val;
 }
-
-#ifndef HAVE_SOUND
-
-/* Dummy functions for when we don't have a sound device; should never be
-   called, so just abort if they are */
-int sound_lowlevel_init(const char *device,int *freqptr,int *stereoptr)
-{
-fuse_abort();
-return 0;
-}
-
-void sound_lowlevel_end( void )
-{
-fuse_abort();
-}
-
-void sound_lowlevel_frame(unsigned char *data,int len)
-{
-fuse_abort();
-}
-
-#endif
