@@ -1,7 +1,7 @@
 /* tape.h: tape handling routines
    Copyright (c) 1999-2004 Philip Kendall
 
-   $Id: tape.h,v 1.22 2004/01/04 20:27:57 pak21 Exp $
+   $Id: tape.h,v 1.23 2004/04/11 10:08:23 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 int tape_init( void );
 
 int tape_open( const char *filename, int autoload );
-int tape_open_default_autoload( const char *filename );
+int tape_open_default_autoload( const char *filename, void *data );
 
 int
 tape_read_buffer( unsigned char *buffer, size_t length, libspectrum_id_t type,
