@@ -1,7 +1,7 @@
 /* trdos.c: Routines for handling the Betadisk interface
    Copyright (c) 2002-2004 Dmitry Sanarin, Fredrick Meunier, Philip Kendall
 
-   $Id: trdos.c,v 1.21 2004/01/08 17:02:57 pak21 Exp $
+   $Id: trdos.c,v 1.30 2004/06/06 22:31:14 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@
 #include "compat.h"
 #include "event.h"
 #include "machine.h"
+#include "memory.h"
 #include "spectrum.h"
 #include "trdos.h"
 #include "ui/ui.h"
@@ -126,7 +127,7 @@ typedef struct
 
 #endif				/* #ifdef WORDS_BIGENDIAN */
 
-# define CurrentDiscNum (last_vg93_system & 3)
+# define CurrentDiscNum ( trdos_system_register & 3 )
 # define CurrentDisk discs[CurrentDiscNum]
 
 static int busy = 0;
@@ -145,13 +146,13 @@ static unsigned int toread_position = 0;
 static libspectrum_byte six_bytes[6];
 
 static int vg_spin;
-static int vg_direction; /* 0 - spindlewards 1 - rimwards */
-static libspectrum_byte vg_reg_trk; /* The last byte sent to VG93 track reg */
-static libspectrum_byte vg_reg_sec; /* The last byte sent to VG93 sector reg */
-static libspectrum_byte vg_reg_dat; /* The last byte sent to VG93 data reg */
+static int trdos_direction; /* 0 - spindlewards 1 - rimwards */
+static libspectrum_byte trdos_status_register; /* Betadisk status register */
+static libspectrum_byte trdos_track_register;  /* FDC track register */
+static libspectrum_byte trdos_sector_register; /* FDC sector register */
+static libspectrum_byte trdos_data_register;   /* FDC data register */
 static libspectrum_byte vg_portFF_in;
-static libspectrum_byte last_vg93_system = 0; /* Last byte sent to VG93 system
-						 port */
+static libspectrum_byte trdos_system_register; /* FDC system register */
 
 union
 {
@@ -210,6 +211,67 @@ trdos_end( void )
   trdos_available = 0;
 }
 
+void
+trdos_page( void )
+{
+  trdos_active = 1;
+  machine_current->ram.romcs = 1;
+  machine_current->memory_map();
+}
+
+void
+trdos_unpage( void )
+{
+  trdos_active = 0;
+  machine_current->ram.romcs = 0;
+  machine_current->memory_map();
+}
+
+void
+trdos_memory_map( void )
+{
+  memory_map_read[0] = memory_map_write[0] = memory_map_romcs[0];
+  memory_map_read[1] = memory_map_write[1] = memory_map_romcs[1];
+}
+  
+int
+trdos_from_snapshot( libspectrum_snap *snap, int capabilities )
+{
+  if( !capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_TRDOS_DISK ) return 0;
+
+  trdos_active = libspectrum_snap_beta_paged( snap );
+  
+  if( trdos_active ) {
+    trdos_page();
+  } else {
+    trdos_unpage();
+  }
+
+  trdos_direction = libspectrum_snap_beta_direction( snap );
+
+  trdos_cr_write ( 0x001f, libspectrum_snap_beta_status( snap ) );
+  trdos_tr_write ( 0x003f, libspectrum_snap_beta_track ( snap ) );
+  trdos_sec_write( 0x005f, libspectrum_snap_beta_sector( snap ) );
+  trdos_dr_write ( 0x007f, libspectrum_snap_beta_data  ( snap ) );
+  trdos_sp_write ( 0x00ff, libspectrum_snap_beta_system( snap ) );
+  
+  return 0;
+}
+
+int
+trdos_to_snapshot( libspectrum_snap *snap )
+{
+  libspectrum_snap_set_beta_paged( snap, trdos_active );
+  libspectrum_snap_set_beta_direction( snap, trdos_direction );
+  libspectrum_snap_set_beta_status( snap, trdos_status_register );
+  libspectrum_snap_set_beta_track ( snap, trdos_system_register );
+  libspectrum_snap_set_beta_sector( snap, trdos_track_register );
+  libspectrum_snap_set_beta_data  ( snap, trdos_sector_register );
+  libspectrum_snap_set_beta_system( snap, trdos_data_register );
+
+  return 0;
+}
+
 static
 void trdos_update_index_impulse( void )
 {
@@ -226,8 +288,12 @@ void trdos_update_index_impulse( void )
 }
 
 libspectrum_byte
-trdos_sr_read( libspectrum_word port GCC_UNUSED )
+trdos_sr_read( libspectrum_word port GCC_UNUSED, int *attached )
 {
+  if( !trdos_active ) return 0xff;
+
+  *attached = 1;
+
   trdos_update_index_impulse();
 
   if ( !CurrentDisk.disc_ready ) {
@@ -238,47 +304,62 @@ trdos_sr_read( libspectrum_word port GCC_UNUSED )
 }
 
 libspectrum_byte
-trdos_tr_read( libspectrum_word port GCC_UNUSED )
+trdos_tr_read( libspectrum_word port GCC_UNUSED, int *attached )
 {
+  if( !trdos_active ) return 0xff;
+
+  *attached = 1;
+
   trdos_update_index_impulse();
 
-  return( vg_reg_trk );
+  return trdos_track_register;
 }
 
 void
 trdos_tr_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
 {
-  vg_reg_trk = b;
+  if( !trdos_active ) return;
+
+  trdos_track_register = b;
 }
 
 libspectrum_byte
-trdos_sec_read( libspectrum_word port GCC_UNUSED )
+trdos_sec_read( libspectrum_word port GCC_UNUSED, int *attached )
 {
+  if( !trdos_active ) return 0xff;
+
+  *attached = 1;
+
   trdos_update_index_impulse();
 
-  return( vg_reg_sec );
+  return trdos_sector_register;
 }
 
 void
 trdos_sec_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
 {
-  vg_reg_sec = b;
+  if( !trdos_active ) return;
+
+  trdos_sector_register = b;
 }
 
 libspectrum_byte
-trdos_dr_read( libspectrum_word port GCC_UNUSED )
+trdos_dr_read( libspectrum_word port GCC_UNUSED, int *attached )
 {
+  if( !trdos_active ) return 0xff;
+
+  *attached = 1;
+
   trdos_update_index_impulse();
 
-  if ( toread_position >= toread_num )
-    return ( vg_reg_dat );
+  if( toread_position >= toread_num ) return trdos_data_register;
 
-  vg_reg_dat = toread[toread_position];
+  trdos_data_register = toread[toread_position];
 
   if ( ( ( toread_position & 0x00ff ) == 0 ) && toread_position != 0 )
-    vg_reg_sec++;
+    trdos_sector_register++;
 
-  if ( vg_reg_sec > 16 ) vg_reg_sec = 1;
+  if( trdos_sector_register > 16 ) trdos_sector_register = 1;
 
   toread_position++;
 
@@ -293,13 +374,15 @@ trdos_dr_read( libspectrum_word port GCC_UNUSED )
     vg_rs.bit.b1 = 1; /*  DRQ copy */
   }
 
-  return( vg_reg_dat );
+  return trdos_data_register;
 }
 
 void
 trdos_dr_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
 {
-  vg_reg_dat = b;
+  if( !trdos_active ) return;
+
+  trdos_data_register = b;
 
   if( towrite == 0 ) return;
 
@@ -326,8 +409,12 @@ trdos_dr_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
 }
 
 libspectrum_byte
-trdos_sp_read( libspectrum_word port GCC_UNUSED )
+trdos_sp_read( libspectrum_word port GCC_UNUSED, int *attached )
 {
+  if( !trdos_active ) return 0xff;
+
+  *attached = 1;
+
   trdos_update_index_impulse();
 
   if ( busy == 1 ) {
@@ -340,7 +427,9 @@ trdos_sp_read( libspectrum_word port GCC_UNUSED )
 void
 trdos_sp_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
 {
-  last_vg93_system = b;
+  if( !trdos_active ) return;
+
+  trdos_system_register = b;
 }
 
 static int
@@ -567,18 +656,17 @@ vg_seek_delay( libspectrum_byte dst_track )
 {
   int error;
 
-  if ( ( dst_track - CurrentDisk.trk ) > 0 )
-    vg_direction = 1;
-  if ( ( dst_track - CurrentDisk.trk ) < 0 )
-    vg_direction = 0;
+  if( ( dst_track - CurrentDisk.trk ) > 0 ) trdos_direction = 1;
+  if( ( dst_track - CurrentDisk.trk ) < 0 ) trdos_direction = 0;
 
   busy = 1;
 
+  /* FIXME: is this code really what was meant? */
   if ( !CurrentDisk.disc_ready ) vg_portFF_in = 0x80;
   else vg_portFF_in = 0x80;
 
   CurrentDisk.trk = dst_track;
-  vg_reg_trk = dst_track;
+  trdos_track_register = dst_track;
 
   /* schedule event */
   error = event_add( tstates +
@@ -608,11 +696,11 @@ trdos_cr_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
 {
   int error;
 
-  if ( last_vg93_system & 0x10 )
-    side = 0;
-  else
-    side = 1;
+  if( !trdos_active ) return;
 
+  trdos_status_register = b;
+
+  side = trdos_system_register & 0x10 ? 0 : 1;
 
   if ( (b & 0xF0) == 0xD0 ) { /*  interrupt */
     vg_portFF_in = 0x80;
@@ -631,7 +719,7 @@ trdos_cr_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
   }
 
   if ( (b & 0xF0) == 0x10 ) { /*  seek track */
-    vg_seek_delay( vg_reg_dat );
+    vg_seek_delay( trdos_data_register );
     vg_rs.bit.b5 = b & 8 ? 1 : 0;
     vg_setFlagsSeeks();
     if ( b & 8 ) vg_spin = 1;
@@ -640,15 +728,15 @@ trdos_cr_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
   }
 
   if ( (b & 0xE0) == 0x40 ) { /*  fwd */
-    vg_direction = 1;
+    trdos_direction = 1;
     b = 0x20; /*  step */
   }
   if ( (b & 0xE0) == 0x60 ) { /*  back */
-    vg_direction = 0;
+    trdos_direction = 0;
     b = 0x20; /*  step */
   }
   if ( (b & 0xE0) == 0x20 ) { /*  step */
-    if ( vg_direction )
+    if ( trdos_direction )
       vg_seek_delay( ++CurrentDisk.trk );
     else
       vg_seek_delay( --CurrentDisk.trk );
@@ -669,22 +757,25 @@ trdos_cr_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
       vg_rs.byte = 0x90;
       vg_portFF_in = 0x80; 
 
-      ui_error( UI_ERROR_INFO, "No disk in drive %c", 'A' + CurrentDiscNum );
+      ui_error( UI_ERROR_WARNING, "No disk in drive %c",
+		'A' + CurrentDiscNum );
 
       return;
     }
 
-    if ( ( vg_reg_sec == 0 ) || ( vg_reg_sec > 16 ) ) {
+    if( ( trdos_sector_register == 0 ) || ( trdos_sector_register > 16 ) ) {
       vg_rs.byte |= 0x10; /*  sector not found */
       vg_portFF_in = 0x80;
 
-      ui_error( UI_ERROR_INFO, "((vg_reg_sec==0)||(vg_reg_sec>16))" );
+      ui_error( UI_ERROR_ERROR,
+		"Attempt to read TRDOS sector with sector register 0x%02x",
+		trdos_sector_register );
 
       return;
     }
 
-    pointer = ( CurrentDisk.trk * 2 + side ) * 256 * 16 + ( vg_reg_sec - 1 ) *
-              256;
+    pointer = ( CurrentDisk.trk * 2 + side ) * 256 * 16 +
+              ( trdos_sector_register - 1 ) * 256;
 
     if( lseek( CurrentDisk.fd, pointer, SEEK_SET) == -1 ) {
       ui_error( UI_ERROR_ERROR,
@@ -699,15 +790,21 @@ trdos_cr_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
 
     if ( b & 0x10 ) {
       ui_error( UI_ERROR_ERROR,
-                "Unimplemented multi sector read:read vg_reg_sec=%d",
-                vg_reg_sec );
+                "Unimplemented TRDOS multisector read: sector register = %d",
+                trdos_sector_register );
 
-      read( CurrentDisk.fd, track, 256 * ( 17 - vg_reg_sec ) );
+      read( CurrentDisk.fd, track, 256 * ( 17 - trdos_sector_register ) );
 
       lseek( CurrentDisk.fd,
-	     -256 * ( ( 17 - vg_reg_sec ) + ( vg_reg_sec - 1 ) ),
+
+	     /* This line used to say:
+	        -256 * ( ( 17 - vg_reg_sec ) + ( vg_reg_sec - 1 ) ),
+		which is fairly nonsensical */
+             -256 * 16,
+
              SEEK_CUR );
-      read( CurrentDisk.fd, track, 256 * ( vg_reg_sec - 1 ) );
+
+      read( CurrentDisk.fd, track, 256 * ( trdos_sector_register - 1 ) );
  
       toread = track;
       toread_num = 256 * ( 16 );
@@ -746,7 +843,7 @@ trdos_cr_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
 
     six_bytes[0] = CurrentDisk.trk;
     six_bytes[1] = 0; 
-    six_bytes[2] = vg_reg_sec;
+    six_bytes[2] = trdos_sector_register;
     six_bytes[3] = 1;
     six_bytes[4] = 0; /*  todo : crc !!! */
     six_bytes[5] = 0; /*  todo : crc !!! */
@@ -787,17 +884,19 @@ trdos_cr_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
       return;
     }
 
-    if ( ( vg_reg_sec == 0 ) || ( vg_reg_sec > 16 ) ) {
+    if ( ( trdos_sector_register == 0 ) || ( trdos_sector_register > 16 ) ) {
       vg_rs.byte |= 0x10; /*  sector not found */
       vg_portFF_in = 0x80;
 
-      ui_error( UI_ERROR_ERROR, "((vg_reg_sec==0)||(vg_reg_sec>16))" );
+      ui_error( UI_ERROR_ERROR,
+		"Attempt to write TRDOS sector with sector register 0x%02x",
+		trdos_sector_register );
 
       return;
     }
 
     towriteaddr = ( CurrentDisk.trk * 2 + side ) * 256 * 16
-                    + ( vg_reg_sec - 1 ) * 256;
+                    + ( trdos_sector_register - 1 ) * 256;
     towrite = 256;
 
     vg_spin = 0;
