@@ -1,7 +1,7 @@
 /* trdos.h: Routines for handling the Betadisk interface
-   Copyright (c) 2003 Fredrick Meunier, Philip Kendall
+   Copyright (c) 2003-2004 Fredrick Meunier, Philip Kendall
 
-   $Id: trdos.h,v 1.6 2004/01/08 17:02:57 pak21 Exp $
+   $Id: trdos.h,v 1.11 2004/06/06 22:31:14 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -40,20 +40,27 @@ void trdos_reset( void );
 
 void trdos_end( void );
 
+void trdos_page( void );
+void trdos_unpage( void );
+void trdos_memory_map( void );
+
+int trdos_from_snapshot( libspectrum_snap *snap, int capabilities );
+int trdos_to_snapshot( libspectrum_snap *snap );
+
 void trdos_cr_write( libspectrum_word port, libspectrum_byte b );
 
-libspectrum_byte trdos_sr_read( libspectrum_word port );
+libspectrum_byte trdos_sr_read( libspectrum_word port, int *attached );
 
-libspectrum_byte trdos_tr_read( libspectrum_word port );
+libspectrum_byte trdos_tr_read( libspectrum_word port, int *attached );
 void trdos_tr_write( libspectrum_word port, libspectrum_byte b );
 
-libspectrum_byte trdos_sec_read( libspectrum_word port );
+libspectrum_byte trdos_sec_read( libspectrum_word port, int *attached );
 void trdos_sec_write( libspectrum_word port, libspectrum_byte b );
 
-libspectrum_byte trdos_dr_read( libspectrum_word port );
+libspectrum_byte trdos_dr_read( libspectrum_word port, int *attached );
 void trdos_dr_write( libspectrum_word port, libspectrum_byte b );
 
-libspectrum_byte trdos_sp_read( libspectrum_word port );
+libspectrum_byte trdos_sp_read( libspectrum_word port, int *attached );
 void trdos_sp_write( libspectrum_word port, libspectrum_byte b );
 
 typedef enum trdos_drive_number {
