@@ -1,7 +1,7 @@
 /* browse.c: tape browser dialog box
    Copyright (c) 2002-2004 Philip Kendall
 
-   $Id: browse.c,v 1.12.2.1 2004/01/30 16:16:19 pak21 Exp $
+   $Id: browse.c,v 1.16 2004/04/11 10:08:24 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #ifdef UI_GTK		/* Use this file iff we're using GTK+ */
 
 #include <stdlib.h>
+#include <string.h>
 
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
@@ -60,7 +61,8 @@ static GtkWidget
 static int dialog_created;	/* Have we created the dialog box yet? */
 
 void
-gtk_tape_browse( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+menu_media_tape_browse( GtkWidget *widget GCC_UNUSED,
+			gpointer data GCC_UNUSED )
 {
   /* Firstly, stop emulation */
   fuse_emulation_pause();
@@ -201,8 +203,8 @@ select_row( GtkCList *clist, gint row, gint column GCC_UNUSED,
 {
   int current_block;
 
-  /* Ignore events which aren't double-clicks */
-  if( event->type != GDK_2BUTTON_PRESS ) return;
+  /* Ignore events which aren't double-clicks or select-via-keyboard */
+  if( event && event->type != GDK_2BUTTON_PRESS ) return;
 
   /* Don't do anything if the current block was clicked on */
   current_block = tape_get_current_block();
