@@ -1,7 +1,7 @@
 /* debugger.c: the GTK+ debugger
-   Copyright (c) 2002-2003 Philip Kendall
+   Copyright (c) 2002-2004 Philip Kendall
 
-   $Id: debugger.c,v 1.54 2004/01/08 23:19:32 pak21 Exp $
+   $Id: debugger.c,v 1.68 2004/07/09 13:09:11 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -43,9 +43,12 @@
 #include "machine.h"
 #include "memory.h"
 #include "scld.h"
+#include "settings.h"
 #include "ui/ui.h"
+#include "ula.h"
 #include "z80/z80.h"
 #include "z80/z80_macros.h"
+#include "zxcf.h"
 
 /* The various debugger panes */
 typedef enum debugger_pane {
@@ -53,6 +56,7 @@ typedef enum debugger_pane {
   DEBUGGER_PANE_BEGIN = 1,	/* Start marker */
 
   DEBUGGER_PANE_REGISTERS = DEBUGGER_PANE_BEGIN,
+  DEBUGGER_PANE_MEMORYMAP,
   DEBUGGER_PANE_BREAKPOINTS,
   DEBUGGER_PANE_DISASSEMBLY,
   DEBUGGER_PANE_STACK,
@@ -70,6 +74,7 @@ static int create_menu_bar( GtkBox *parent, GtkAccelGroup *accel_group );
 static void toggle_display( gpointer callback_data, guint callback_action,
 			    GtkWidget *widget );
 static int create_register_display( GtkBox *parent, gtkui_font font );
+static int create_memory_map( GtkBox *parent );
 static int create_breakpoints( GtkBox *parent );
 static int create_disassembly( GtkBox *parent, gtkui_font font );
 static int create_stack_display( GtkBox *parent, gtkui_font font );
@@ -82,6 +87,8 @@ static int create_command_entry( GtkBox *parent, GtkAccelGroup *accel_group );
 static int create_buttons( GtkContainer *parent, GtkAccelGroup *accel_group );
 
 static int activate_debugger( void );
+static int update_memory_map( void );
+static int update_breakpoints( void );
 static int update_disassembly( void );
 static int update_events( void );
 static void add_event( gpointer data, gpointer user_data );
@@ -102,6 +109,8 @@ static GtkWidget *dialog,		/* The debugger dialog box */
   *continue_button, *break_button,	/* Two of its buttons */
   *register_display,			/* The register display */
   *registers[18],			/* Individual registers */
+  *memorymap,				/* The memory map display */
+  *map_label[8][4],			/* Labels in the memory map */
   *breakpoints,				/* The breakpoint display */
   *disassembly_box,			/* A box to hold the disassembly */
   *disassembly,				/* The actual disassembly widget */
@@ -127,6 +136,7 @@ static GtkItemFactoryEntry menu_data[] = {
 
   { "/_View", NULL, NULL, 0, "<Branch>" },
   { "/View/_Registers", NULL, toggle_display, DEBUGGER_PANE_REGISTERS, "<ToggleItem>" },
+  { "/View/_Memory Map", NULL, toggle_display, DEBUGGER_PANE_MEMORYMAP, "<ToggleItem>" },
   { "/View/_Breakpoints", NULL, toggle_display, DEBUGGER_PANE_BREAKPOINTS, "<ToggleItem>" },
   { "/View/_Disassembly", NULL, toggle_display, DEBUGGER_PANE_DISASSEMBLY, "<ToggleItem>" },
   { "/View/_Stack", NULL, toggle_display, DEBUGGER_PANE_STACK, "<ToggleItem>" },
@@ -137,6 +147,18 @@ static GtkItemFactoryEntry menu_data[] = {
 static guint menu_data_count =
   sizeof( menu_data ) / sizeof( GtkItemFactoryEntry );
 
+static const char*
+format_8_bit( void )
+{
+  return debugger_output_base == 10 ? "%3d" : "0x%02X";
+}
+
+static const char*
+format_16_bit( void )
+{
+  return debugger_output_base == 10 ? "%5d" : "0x%04X";
+}
+
 int
 ui_debugger_activate( void )
 {
@@ -187,6 +209,7 @@ get_pane_menu_item( debugger_pane pane )
 
   switch( pane ) {
   case DEBUGGER_PANE_REGISTERS: path = "/View/Registers"; break;
+  case DEBUGGER_PANE_MEMORYMAP: path = "/View/Memory Map"; break;
   case DEBUGGER_PANE_BREAKPOINTS: path = "/View/Breakpoints"; break;
   case DEBUGGER_PANE_DISASSEMBLY: path = "/View/Disassembly"; break;
   case DEBUGGER_PANE_STACK: path = "/View/Stack"; break;
@@ -215,6 +238,7 @@ get_pane( debugger_pane pane )
 {
   switch( pane ) {
   case DEBUGGER_PANE_REGISTERS: return register_display;
+  case DEBUGGER_PANE_MEMORYMAP: return memorymap;
   case DEBUGGER_PANE_BREAKPOINTS: return breakpoints;
   case DEBUGGER_PANE_DISASSEMBLY: return disassembly_box;
   case DEBUGGER_PANE_STACK: return stack;
@@ -242,7 +266,7 @@ static int
 create_dialog( void )
 {
   int error;
-  GtkWidget *hbox, *vbox;
+  GtkWidget *hbox, *vbox, *hbox2;
   GtkAccelGroup *accel_group;
   debugger_pane i;
 
@@ -266,7 +290,7 @@ create_dialog( void )
 			   accel_group );
   if( error ) return error;
 
-  /* A couple of boxes to contain the things we want to display */
+  /* Some boxes to contain the things we want to display */
   hbox = gtk_hbox_new( FALSE, 0 );
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG( dialog )->vbox ), hbox,
 		      TRUE, TRUE, 5 );
@@ -274,10 +298,15 @@ create_dialog( void )
   vbox = gtk_vbox_new( FALSE, 5 );
   gtk_box_pack_start( GTK_BOX( hbox ), vbox, TRUE, TRUE, 5 );
 
+  hbox2 = gtk_hbox_new( FALSE, 5 );
+  gtk_box_pack_start_defaults( GTK_BOX( vbox ), hbox2 );
+
   /* The main display areas */
-  error = create_register_display( GTK_BOX( vbox ), font );
+  error = create_register_display( GTK_BOX( hbox2 ), font );
   if( error ) return error;
 
+  error = create_memory_map( GTK_BOX( hbox2 ) ); if( error ) return error;
+
   error = create_breakpoints( GTK_BOX( vbox ) ); if( error ) return error;
 
   error = create_disassembly( GTK_BOX( hbox ), font );
@@ -325,7 +354,7 @@ create_menu_bar( GtkBox *parent, GtkAccelGroup *accel_group )
 				 NULL);
   menu_bar = gtk_item_factory_get_widget( menu_factory, "<main>" );
 
-  gtk_box_pack_start_defaults( parent, menu_bar );
+  gtk_box_pack_start( parent, menu_bar, FALSE, FALSE, 0 );
   
   return 0;
 }
@@ -363,6 +392,41 @@ create_register_display( GtkBox *parent, gtkui_font font )
   return 0;
 }
 
+static int
+create_memory_map( GtkBox *parent )
+{
+  GtkWidget *table, *label;
+  size_t i, j;
+
+  memorymap = gtk_frame_new( "Memory Map" );
+  gtk_box_pack_start( parent, memorymap, FALSE, FALSE, 0 );
+
+  table = gtk_table_new( 9, 4, FALSE );
+  gtk_container_add( GTK_CONTAINER( memorymap ), table );
+
+  label = gtk_label_new( "Address" );
+  gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 0, 1, 0, 0, 2, 2 );
+
+  label = gtk_label_new( "Source" );
+  gtk_table_attach( GTK_TABLE( table ), label, 1, 2, 0, 1, 0, 0, 2, 2 );
+
+  label = gtk_label_new( "W?" );
+  gtk_table_attach( GTK_TABLE( table ), label, 2, 3, 0, 1, 0, 0, 2, 2 );
+
+  label = gtk_label_new( "C?" );
+  gtk_table_attach( GTK_TABLE( table ), label, 3, 4, 0, 1, 0, 0, 2, 2 );
+
+  for( i = 0; i < 8; i++ ) {
+    for( j = 0; j < 4; j++ ) {
+      map_label[i][j] = gtk_label_new( "" );
+      gtk_table_attach( GTK_TABLE( table ), map_label[i][j],
+			j, j + 1, 8 - i, 9 - i, 0, 0, 2, 2 );
+    }
+  }
+
+  return 0;
+}
+
 static int
 create_breakpoints( GtkBox *parent )
 {
@@ -439,17 +503,18 @@ stack_click( GtkCList *clist, gint row, gint column,
   libspectrum_word address, destination;
   int error;
 
-  /* Ignore events which aren't a double-click */
-  if( event->type != GDK_2BUTTON_PRESS ) return;
+  /* Ignore events which aren't double-clicks or select-via-keyboard */
+  if( event && event->type != GDK_2BUTTON_PRESS ) return;
 
   address = SP + ( 19 - row ) * 2;
 
   destination =
     readbyte_internal( address ) + readbyte_internal( address + 1 ) * 0x100;
 
-  error = debugger_breakpoint_add( DEBUGGER_BREAKPOINT_TYPE_EXECUTE,
-				   -1, destination, 0,
-				   DEBUGGER_BREAKPOINT_LIFE_ONESHOT, NULL );
+  error = debugger_breakpoint_add_address(
+    DEBUGGER_BREAKPOINT_TYPE_EXECUTE, -1, destination, 0,
+    DEBUGGER_BREAKPOINT_LIFE_ONESHOT, NULL
+  );
   if( error ) return;
 
   debugger_run();
@@ -481,8 +546,8 @@ events_click( GtkCList *clist, gint row, gint column,
   gchar *buffer;
   libspectrum_dword tstates;
 
-  /* Ignore events which aren't a double-click */
-  if( event->type != GDK_2BUTTON_PRESS ) return;
+  /* Ignore events which aren't double-clicks or select-via-keyboard */
+  if( event && event->type != GDK_2BUTTON_PRESS ) return;
 
   got_text = gtk_clist_get_text( clist, row, 0, &buffer );
   if( !got_text ) {
@@ -491,10 +556,10 @@ events_click( GtkCList *clist, gint row, gint column,
   }
 
   tstates = atoi( buffer );
-
-  error = debugger_breakpoint_add( DEBUGGER_BREAKPOINT_TYPE_TIME,
-				   -1, tstates, 0,
-				   DEBUGGER_BREAKPOINT_LIFE_ONESHOT, NULL );
+  error = debugger_breakpoint_add_time(
+    DEBUGGER_BREAKPOINT_TYPE_TIME, tstates, 0,
+    DEBUGGER_BREAKPOINT_LIFE_ONESHOT, NULL
+  );
   if( error ) return;
 
   debugger_run();
@@ -582,12 +647,8 @@ ui_debugger_update( void )
 {
   size_t i;
   char buffer[1024], format_string[1024];
-  gchar *breakpoint_text[6] = { &buffer[  0], &buffer[ 40], &buffer[80],
-			        &buffer[120], &buffer[160], &buffer[200] },
-    *disassembly_text[2] = { &buffer[0], &buffer[40] };
+  gchar *disassembly_text[2] = { &buffer[0], &buffer[40] };
   libspectrum_word address;
-  const char *format_16_bit, *format_8_bit;
-  GSList *ptr;
   int capabilities; size_t length;
   int error;
 
@@ -606,22 +667,16 @@ ui_debugger_update( void )
 
   if( !dialog_created ) return 0;
 
-  if( debugger_output_base == 10 ) {
-    format_16_bit = "%5d"; format_8_bit = "%3d";
-  } else {
-    format_16_bit = "0x%04X"; format_8_bit = "0x%02X";
-  }
-
   for( i = 0; i < 12; i++ ) {
     snprintf( buffer, 5, "%3s ", register_name[i] );
-    snprintf( &buffer[4], 76, format_16_bit, *value_ptr[i] );
+    snprintf( &buffer[4], 76, format_16_bit(), *value_ptr[i] );
     gtk_label_set_text( GTK_LABEL( registers[i] ), buffer );
   }
 
-  strcpy( buffer, "  I   " ); snprintf( &buffer[6], 76, format_8_bit, I );
+  strcpy( buffer, "  I   " ); snprintf( &buffer[6], 76, format_8_bit(), I );
   gtk_label_set_text( GTK_LABEL( registers[12] ), buffer );
   strcpy( buffer, "  R   " );
-  snprintf( &buffer[6], 80, format_8_bit, ( R & 0x7f ) | ( R7 & 0x80 ) );
+  snprintf( &buffer[6], 80, format_8_bit(), ( R & 0x7f ) | ( R7 & 0x80 ) );
   gtk_label_set_text( GTK_LABEL( registers[13] ), buffer );
 
   snprintf( buffer, 80, "T-states %5d", tstates );
@@ -636,45 +691,119 @@ ui_debugger_update( void )
 
   capabilities = libspectrum_machine_capabilities( machine_current->machine );
 
-  sprintf( format_string, "   ULA %s", format_8_bit );
-  snprintf( buffer, 1024, format_string, spectrum_last_ula );
+  sprintf( format_string, "   ULA %s", format_8_bit() );
+  snprintf( buffer, 1024, format_string, ula_last_byte() );
 
   if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_AY ) {
-    sprintf( format_string, "\n    AY %s", format_8_bit );
+    sprintf( format_string, "\n    AY %s", format_8_bit() );
     length = strlen( buffer );
     snprintf( &buffer[length], 1024-length, format_string,
 	      machine_current->ay.current_register );
   }
 
   if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_128_MEMORY ) {
-    sprintf( format_string, "\n128Mem %s", format_8_bit );
+    sprintf( format_string, "\n128Mem %s", format_8_bit() );
     length = strlen( buffer );
     snprintf( &buffer[length], 1024-length, format_string,
 	      machine_current->ram.last_byte );
   }
 
   if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_PLUS3_MEMORY ) {
-    sprintf( format_string, "\n+3 Mem %s", format_8_bit );
+    sprintf( format_string, "\n+3 Mem %s", format_8_bit() );
     length = strlen( buffer );
     snprintf( &buffer[length], 1024-length, format_string,
 	      machine_current->ram.last_byte2 );
   }
 
-  if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_TIMEX_MEMORY ) {
-    sprintf( format_string, "\nTmxDec %s", format_8_bit );
+  if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_TIMEX_VIDEO  ||
+      capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_TIMEX_MEMORY ||
+      capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_SE_MEMORY       ) {
+    sprintf( format_string, "\nTmxDec %s", format_8_bit() );
     length = strlen( buffer );
     snprintf( &buffer[length], 1024-length, format_string,
 	      scld_last_dec.byte );
   }
 
-  if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_TIMEX_VIDEO ) {
-    sprintf( format_string, "\nTmxHsr %s", format_8_bit );
+  if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_TIMEX_MEMORY ||
+      capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_SE_MEMORY       ) {
+    sprintf( format_string, "\nTmxHsr %s", format_8_bit() );
     length = strlen( buffer );
     snprintf( &buffer[length], 1024-length, format_string, scld_last_hsr );
   }
 
+  if( settings_current.zxcf_active ) {
+    sprintf( format_string, "\n  ZXCF %s", format_8_bit() );
+    length = strlen( buffer );
+    snprintf( &buffer[length], 1024-length, format_string,
+	      zxcf_last_memctl() );
+  }
+
   gtk_label_set_text( GTK_LABEL( registers[17] ), buffer );
 
+  /* Update the memory map display */
+  error = update_memory_map(); if( error ) return error;
+
+  error = update_breakpoints(); if( error ) return error;
+
+  /* Update the disassembly */
+  error = update_disassembly(); if( error ) return error;
+
+  /* And the stack display */
+  gtk_clist_freeze( GTK_CLIST( stack ) );
+  gtk_clist_clear( GTK_CLIST( stack ) );
+
+  for( i = 0, address = SP + 38; i < 20; i++, address -= 2 ) {
+    
+    libspectrum_word contents = readbyte_internal( address ) +
+				0x100 * readbyte_internal( address + 1 );
+
+    snprintf( disassembly_text[0], 40, format_16_bit(), address );
+    snprintf( disassembly_text[1], 40, format_16_bit(), contents );
+    gtk_clist_append( GTK_CLIST( stack ), disassembly_text );
+  }
+
+  gtk_clist_thaw( GTK_CLIST( stack ) );
+
+  /* And the events display */
+  error = update_events(); if( error ) return error;
+
+  return 0;
+}
+
+static int
+update_memory_map( void )
+{
+  size_t i;
+  char buffer[ 40 ];
+
+  for( i = 0; i < 8; i++ ) {
+
+    snprintf( buffer, 40, format_16_bit(), (unsigned)i * 0x2000 );
+    gtk_label_set_text( GTK_LABEL( map_label[i][0] ), buffer );
+
+    snprintf( buffer, 40, "%s %d", memory_bank_name( &memory_map_read[i] ),
+	      memory_map_read[i].page_num );
+    gtk_label_set_text( GTK_LABEL( map_label[i][1] ), buffer );
+
+    gtk_label_set_text( GTK_LABEL( map_label[i][2] ),
+			memory_map_read[i].writable ? "Y" : "N" );
+
+    gtk_label_set_text( GTK_LABEL( map_label[i][3] ),
+			memory_map_read[i].contended ? "Y" : "N" );
+  }
+
+  return 0;
+}
+
+static int
+update_breakpoints( void )
+{
+  gchar buffer[ 1024 ],
+    *breakpoint_text[6] = { &buffer[  0], &buffer[ 40], &buffer[80],
+			    &buffer[120], &buffer[160], &buffer[200] };
+  GSList *ptr;
+  char format_string[ 1024 ], page[ 1024 ];
+
   /* Create the breakpoint list */
   gtk_clist_freeze( GTK_CLIST( breakpoints ) );
   gtk_clist_clear( GTK_CLIST( breakpoints ) );
@@ -687,15 +816,33 @@ ui_debugger_update( void )
     snprintf( breakpoint_text[1], 40, "%s",
 	      debugger_breakpoint_type_text[ bp->type ] );
 
-    if( bp->type == DEBUGGER_BREAKPOINT_TYPE_TIME ) {
-      snprintf( breakpoint_text[2], 40, "%5d", bp->value );
-    } else {
-      if( bp->page == -1 ) {
-	snprintf( breakpoint_text[2], 40, format_16_bit, bp->value );
+    switch( bp->type ) {
+
+    case DEBUGGER_BREAKPOINT_TYPE_EXECUTE:
+    case DEBUGGER_BREAKPOINT_TYPE_READ:
+    case DEBUGGER_BREAKPOINT_TYPE_WRITE:
+      if( bp->value.address.page == -1 ) {
+	snprintf( breakpoint_text[2], 40, format_16_bit(),
+		  bp->value.address.offset );
       } else {
-	sprintf( format_string, "%%d:%s", format_16_bit );
-	snprintf( breakpoint_text[2], 40, format_string, bp->page, bp->value );
+	debugger_breakpoint_decode_page( page, 1024, bp->value.address.page );
+	snprintf( format_string, 1024, "%%s:%s", format_16_bit() );
+	snprintf( breakpoint_text[2], 40, format_string, page,
+		  bp->value.address.offset );
       }
+      break;
+
+    case DEBUGGER_BREAKPOINT_TYPE_PORT_READ:
+    case DEBUGGER_BREAKPOINT_TYPE_PORT_WRITE:
+      sprintf( format_string, "%s:%s", format_16_bit(), format_16_bit() );
+      snprintf( breakpoint_text[2], 40, format_string,
+		bp->value.port.mask, bp->value.port.port );
+      break;
+
+    case DEBUGGER_BREAKPOINT_TYPE_TIME:
+      snprintf( breakpoint_text[2], 40, "%5d", bp->value.tstates );
+      break;
+
     }
 
     snprintf( breakpoint_text[3], 40, "%lu", (unsigned long)bp->ignore );
@@ -712,28 +859,6 @@ ui_debugger_update( void )
 
   gtk_clist_thaw( GTK_CLIST( breakpoints ) );
 
-  /* Update the disassembly */
-  error = update_disassembly(); if( error ) return error;
-
-  /* And the stack display */
-  gtk_clist_freeze( GTK_CLIST( stack ) );
-  gtk_clist_clear( GTK_CLIST( stack ) );
-
-  for( i = 0, address = SP + 38; i < 20; i++, address -= 2 ) {
-    
-    libspectrum_word contents = readbyte_internal( address ) +
-				0x100 * readbyte_internal( address + 1 );
-
-    snprintf( disassembly_text[0], 40, format_16_bit, address );
-    snprintf( disassembly_text[1], 40, format_16_bit, contents );
-    gtk_clist_append( GTK_CLIST( stack ), disassembly_text );
-  }
-
-  gtk_clist_thaw( GTK_CLIST( stack ) );
-
-  /* And the events display */
-  error = update_events(); if( error ) return error;
-
   return 0;
 }
 
@@ -744,15 +869,12 @@ update_disassembly( void )
   char buffer[80];
   char *disassembly_text[2] = { &buffer[0], &buffer[40] };
 
-  const char *format_16_bit =
-    ( debugger_output_base == 10 ? "%5d" : "0x%04X" );
-
   gtk_clist_freeze( GTK_CLIST( disassembly ) );
   gtk_clist_clear( GTK_CLIST( disassembly ) );
 
   for( i = 0, address = disassembly_top; i < 20; i++ ) {
 
-    snprintf( disassembly_text[0], 40, format_16_bit, address );
+    snprintf( disassembly_text[0], 40, format_16_bit(), address );
     debugger_disassemble( disassembly_text[1], 40, &length, address );
     address += length;
 
