@@ -1,7 +1,7 @@
 /* gtkui.c: GTK+ routines for dealing with the user interface
    Copyright (c) 2000-2004 Philip Kendall, Russell Marks
 
-   $Id: gtkui.c,v 1.123.2.2 2004/01/31 16:40:58 pak21 Exp $
+   $Id: gtkui.c,v 1.139 2004/05/28 10:04:39 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -38,29 +38,20 @@
 
 #include <libspectrum.h>
 
-#include "dck.h"
 #include "debugger/debugger.h"
-#include "display.h"
-#include "event.h"
 #include "fuse.h"
 #include "gtkinternals.h"
 #include "machine.h"
-#include "options.h"
-#include "pokefinder/pokefinder.h"
+#include "menu.h"
 #include "psg.h"
 #include "rzx.h"
 #include "screenshot.h"
 #include "settings.h"
 #include "snapshot.h"
-#include "specplus3.h"
-#include "spectrum.h"
+#include "machines/specplus3.h"
 #include "tape.h"
 #include "timer.h"
-#include "trdos.h"
 #include "ui/ui.h"
-#include "ui/uidisplay.h"
-#include "ui/scaler/scaler.h"
-#include "utils.h"
 
 /* The main Fuse window */
 GtkWidget *gtkui_window;
@@ -88,7 +79,7 @@ typedef struct gtkui_select_info {
   GtkWidget **buttons;
 
   /* Used by the graphics filter selectors */
-  ui_scaler_available available;
+  scaler_available_fn selector;
   scaler_type selected;
 
 } gtkui_select_info;
@@ -100,140 +91,13 @@ static gboolean gtkui_make_menu(GtkAccelGroup **accel_group,
 
 static gboolean gtkui_delete( GtkWidget *widget, GdkEvent *event,
 			      gpointer data );
-static void gtkui_open(GtkWidget *widget, gpointer data);
-static void gtkui_save(GtkWidget *widget, gpointer data);
 
-static void gtkui_rzx_start( GtkWidget *widget, gpointer data );
-static void gtkui_rzx_start_snap( GtkWidget *widget, gpointer data );
-static void gtkui_rzx_stop( GtkWidget *widget, gpointer data );
-static void gtkui_rzx_play( GtkWidget *widget, gpointer data );
-static int gtkui_open_snap( void );
-
-static void gtkui_psg_start( GtkWidget *widget, gpointer data );
-static void gtkui_psg_stop( GtkWidget *widget, gpointer data );
-
-static void gtkui_open_scr( GtkWidget *widget, gpointer data );
-static void gtkui_save_scr( GtkWidget *widget, gpointer data );
-#ifdef USE_LIBPNG
-static void gtkui_save_screen( GtkWidget *widget, gpointer data );
-#endif				/* #ifdef USE_LIBPNG */
-
-static void gtkui_quit(GtkWidget *widget, gpointer data);
-
-static void select_filter(GtkWidget *widget, gpointer data);
-static void select_filter_done( GtkWidget *widget, gpointer user_data );
-
-#ifdef HAVE_LIB_XML2
-static void save_options( GtkWidget *widget, gpointer data );
-#endif				/* #ifdef HAVE_LIB_XML2 */
-
-static void gtkui_pause( GtkWidget *widget, gpointer data );
-static void gtkui_reset(GtkWidget *widget, gpointer data);
-
-static void gtkui_select(GtkWidget *widget, gpointer data);
-static void gtkui_select_done( GtkWidget *widget, gpointer user_data );
-
-static void gtkui_break( GtkWidget *widget, gpointer data );
-static void gtkui_nmi( GtkWidget *widget, gpointer data );
-
-static void gtkui_tape_open( GtkWidget *widget, gpointer data );
-static void gtkui_tape_play( GtkWidget *widget, gpointer data );
-static void gtkui_tape_rewind( GtkWidget *widget, gpointer data );
-static void gtkui_tape_clear( GtkWidget *widget, gpointer data );
-static void gtkui_tape_write( GtkWidget *widget, gpointer data );
-
-static void disk_open( gpointer data, guint action, GtkWidget *widget );
-static void gtkui_disk_open( specplus3_drive_number drive );
-static void disk_eject( gpointer data, guint action, GtkWidget *widget );
-
-static void cartridge_insert( GtkWidget *widget, gpointer data );
-static void cartridge_eject( GtkWidget *widget, gpointer data );
-
-static void gtkui_help_keyboard( GtkWidget *widget, gpointer data );
+static void menu_options_filter_done( GtkWidget *widget, gpointer user_data );
+static void menu_machine_select_done( GtkWidget *widget, gpointer user_data );
 
 /* Set a menu item (in)active in both the menu bar and the popup menus */
 static int set_menu_item_active( const char *path, int active );
 
-static GtkItemFactoryEntry gtkui_menu_data[] = {
-  { "/File",		        NULL , NULL,                0, "<Branch>"    },
-  { "/File/_Open...",		"F3" , gtkui_open,          0, NULL          },
-  { "/File/_Save Snapshot..." , "F2" , gtkui_save,          0, NULL          },
-  { "/File/_Recording",		NULL , NULL,		    0, "<Branch>"    },
-  { "/File/Recording/_Record...",NULL, gtkui_rzx_start,     0, NULL	     },
-  { "/File/Recording/Record _from snapshot...",
-                                NULL , gtkui_rzx_start_snap,0, NULL          },
-  { "/File/Recording/_Play...", NULL , gtkui_rzx_play,	    0, NULL          },
-  { "/File/Recording/_Stop",    NULL , gtkui_rzx_stop,	    0, NULL          },
-  { "/File/A_Y Logging",        NULL , NULL,                0, "<Branch>"    },
-  { "/File/AY Logging/_Record...",NULL, gtkui_psg_start,    0, NULL          },
-  { "/File/AY Logging/_Stop",   NULL , gtkui_psg_stop,      0, NULL          },
-
-  { "/File/separator",          NULL , NULL,                0, "<Separator>" },
-  { "/File/O_pen SCR Screenshot...", NULL, gtkui_open_scr,  0, NULL          },
-  { "/File/S_ave Screen as SCR...", NULL, gtkui_save_scr,   0, NULL          },
-#ifdef USE_LIBPNG
-  { "/File/Save S_creen as PNG...", NULL,gtkui_save_screen, 0, NULL          },
-#endif				/* #ifdef USE_LIBPNG */
-
-  { "/File/separator",          NULL , NULL,                0, "<Separator>" },
-  { "/File/Loa_d binary data...",NULL, gtkui_load_binary_data, 0, NULL       },
-  { "/File/Save _binary data...",NULL, gtkui_save_binary_data, 0, NULL       },
-
-  { "/File/separator",          NULL , NULL,                0, "<Separator>" },
-  { "/File/E_xit...",	        "F10", gtkui_quit,          0, NULL          },
-
-  { "/Options",			NULL , NULL,                0, "<Branch>"    },
-  { "/Options/_General...",     "F4" , gtkoptions_general,  0, NULL          },
-  { "/Options/_Sound...",	NULL , gtkoptions_sound,    0, NULL          },
-  { "/Options/_RZX...",		NULL , gtkoptions_rzx,      0, NULL          },
-  { "/Options/S_elect ROMs...", NULL , gtkui_roms,          0, NULL          },
-  { "/Options/_Filter...",	NULL , select_filter,	    0, NULL          },
-
-#ifdef HAVE_LIB_XML2
-  { "/Options/separator",       NULL , NULL,                0, "<Separator>" },
-  { "/Options/S_ave",		NULL , save_options,	    0, NULL          },
-#endif				/* #ifdef HAVE_LIB_XML2 */
-
-  { "/Machine",		        NULL , NULL,                0, "<Branch>"    },
-  { "/Machine/_Pause...",       "Pause", gtkui_pause,	    0, NULL          },
-  { "/Machine/_Reset...",       "F5" , gtkui_reset,         0, NULL          },
-  { "/Machine/_Select...",      "F9" , gtkui_select,        0, NULL          },
-  { "/Machine/_Debugger...",	NULL , gtkui_break,	    0, NULL          },
-  { "/Machine/P_oke Finder...",	NULL , gtkui_pokefinder,    0, NULL	     },
-  { "/Machine/_Memory Browser...", NULL, gtkui_memory_browser, 0, NULL       },
-  { "/Machine/_NMI",		NULL , gtkui_nmi,	    0, NULL          },
-
-  { "/Media",			NULL , NULL,                0, "<Branch>"    },
-
-  { "/Media/_Tape",		NULL , NULL,                0, "<Branch>"    },
-  { "/Media/Tape/_Open...",	"F7" , gtkui_tape_open,     0, NULL          },
-  { "/Media/Tape/_Play",	"F8" , gtkui_tape_play,     0, NULL          },
-  { "/Media/Tape/_Browse...",	NULL , gtk_tape_browse,     0, NULL          },
-  { "/Media/Tape/_Rewind",	NULL , gtkui_tape_rewind,   0, NULL          },
-  { "/Media/Tape/_Clear",	NULL , gtkui_tape_clear,    0, NULL          },
-  { "/Media/Tape/_Write...",	"F6" , gtkui_tape_write,    0, NULL          },
-
-  { "/Media/_Disk",		NULL , NULL,		    0, "<Branch>"    },
-  { "/Media/Disk/Drive A:",	NULL , NULL,		    0, "<Branch>"    },
-  { "/Media/Disk/Drive A:/_Insert...", NULL, disk_open,     1, NULL          },
-  { "/Media/Disk/Drive A:/_Eject", NULL, disk_eject,        1, NULL          },
-  { "/Media/Disk/Drive A:/Eject and _write...", NULL, disk_eject, 3, NULL    },
-  { "/Media/Disk/Drive B:",	NULL , NULL,		    0, "<Branch>"    },
-  { "/Media/Disk/Drive B:/_Insert...", NULL, disk_open,     2, NULL          },
-  { "/Media/Disk/Drive B:/_Eject", NULL, disk_eject,        2, NULL          },
-  { "/Media/Disk/Drive B:/Eject and _write...", NULL, disk_eject, 4, NULL    },
-
-  { "/Media/_Cartridge",	NULL , NULL,		    0, "<Branch>"    },
-  { "/Media/Cartridge/_Insert...",
-				NULL , cartridge_insert,    0, NULL          },
-  { "/Media/Cartridge/_Eject",	NULL , cartridge_eject,     0, NULL          },
-
-  { "/Help",			NULL , NULL,		    0, "<Branch>"    },
-  { "/Help/_Keyboard...",	NULL , gtkui_help_keyboard, 0, NULL	     },
-};
-static guint gtkui_menu_data_size =
-  sizeof( gtkui_menu_data ) / sizeof( GtkItemFactoryEntry );
-  
 int
 ui_init( int *argc, char ***argv )
 {
@@ -273,8 +137,8 @@ ui_init( int *argc, char ***argv )
   gtk_container_add(GTK_CONTAINER(gtkui_window), box);
   gtk_widget_show(box);
 
-  gtkui_make_menu(&accel_group, &menu_bar, gtkui_menu_data,
-		  gtkui_menu_data_size);
+  gtkui_make_menu( &accel_group, &menu_bar, gtkui_menu_data,
+		   gtkui_menu_data_size );
 
   gtk_window_add_accel_group( GTK_WINDOW(gtkui_window), accel_group );
   gtk_box_pack_start( GTK_BOX(box), menu_bar, FALSE, FALSE, 0 );
@@ -391,45 +255,95 @@ int ui_end(void)
   return 0;
 }
 
+/* Create a dialog box with the given error message */
+int
+ui_error_specific( ui_error_level severity, const char *message )
+{
+  GtkWidget *dialog, *ok_button, *label, *vbox;
+  GtkAccelGroup *accel_group;
+
+  /* If we don't have a UI yet, we can't output widgets */
+  if( !display_ui_initialised ) return 0;
+
+  /* Create the dialog box */
+  dialog = gtk_dialog_new();
+
+  /* Set the appropriate title */
+  switch( severity ) {
+
+  case UI_ERROR_INFO:
+    gtk_window_set_title( GTK_WINDOW( dialog ), "Fuse - Info" ); break;
+  case UI_ERROR_WARNING:
+    gtk_window_set_title( GTK_WINDOW( dialog ), "Fuse - Warning" ); break;
+  case UI_ERROR_ERROR:
+    gtk_window_set_title( GTK_WINDOW( dialog ), "Fuse - Error" ); break;
+  default:
+    gtk_window_set_title( GTK_WINDOW( dialog ),
+			  "Fuse - (Unknown error level)" );
+    break;
+
+  }
+  
+  /* Add the OK button into the lower half */
+  ok_button = gtk_button_new_with_label( "OK" );
+  gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog )->action_area ),
+		     ok_button );
+
+  /* Create a label with that message */
+  label = gtk_label_new( message );
+
+  /* Make a new vbox for the top part for saner spacing */
+  vbox=gtk_vbox_new( FALSE, 0 );
+  gtk_box_pack_start( GTK_BOX( GTK_DIALOG( dialog )->vbox ),
+                      vbox, TRUE, TRUE, 0 );
+  gtk_container_set_border_width( GTK_CONTAINER( vbox ), 5 );
+  gtk_container_set_border_width( GTK_CONTAINER( GTK_DIALOG( dialog )->action_area ),
+                                  5 );
+
+  /* Put the label in it */
+  gtk_container_add( GTK_CONTAINER( vbox ), label );
+
+  /* Add some ways to finish the dialog box */
+  gtk_signal_connect_object( GTK_OBJECT( ok_button ), "clicked",
+			     GTK_SIGNAL_FUNC( gtk_widget_destroy ),
+			     GTK_OBJECT( dialog ) );
+  gtk_signal_connect( GTK_OBJECT( dialog ), "delete-event",
+		      GTK_SIGNAL_FUNC( gtk_widget_destroy ), (gpointer) NULL );
+
+  accel_group = gtk_accel_group_new();
+  gtk_window_add_accel_group(GTK_WINDOW(dialog), accel_group);
+
+  gtk_widget_add_accelerator( ok_button, "clicked",
+			      accel_group,
+			      GDK_Return, 0, 0 );
+  gtk_widget_add_accelerator( ok_button, "clicked",
+			      accel_group,
+			      GDK_Escape, 0, 0 );
+
+  gtk_widget_show_all( dialog );
+
+  return 0;
+}
+  
 /* The callbacks used by various routines */
 
 /* Called by the main window on a "delete-event" */
 static gboolean
 gtkui_delete( GtkWidget *widget, GdkEvent *event GCC_UNUSED, gpointer data )
 {
-  gtkui_quit( widget, data );
+  menu_file_exit( widget, data );
   return TRUE;
 }
 
-/* Called by the menu when File/Open selected */
-static void
-gtkui_open( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+void
+menu_file_savesnapshot( GtkWidget *widget GCC_UNUSED,
+			gpointer data GCC_UNUSED )
 {
   char *filename;
 
   fuse_emulation_pause();
 
-  filename = gtkui_fileselector_get_filename( "Fuse - Open Spectrum File" );
-  if( !filename ) { fuse_emulation_unpause(); return; }
-
-  utils_open_file( filename, settings_current.auto_load, NULL );
-
-  free( filename );
-
-  display_refresh_all();
-
-  fuse_emulation_unpause();
-}
-
-/* Called by the menu when File/Save selected */
-static void
-gtkui_save( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
-  char *filename;
-
-  fuse_emulation_pause();
-
-  filename = gtkui_fileselector_get_filename( "Fuse - Save Snapshot" );
+  filename = menu_get_filename( "Fuse - Save Snapshot" );
   if( !filename ) { fuse_emulation_unpause(); return; }
 
   snapshot_write( filename );
@@ -439,9 +353,9 @@ gtkui_save( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
   fuse_emulation_unpause();
 }
 
-/* Called when File/Recording/Record selected */
-static void
-gtkui_rzx_start( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+void
+menu_file_recording_record( GtkWidget *widget GCC_UNUSED,
+			    gpointer data GCC_UNUSED )
 {
   char *recording;
 
@@ -449,7 +363,7 @@ gtkui_rzx_start( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 
   fuse_emulation_pause();
 
-  recording = gtkui_fileselector_get_filename( "Fuse - Start Recording" );
+  recording = menu_get_filename( "Fuse - Start Recording" );
   if( !recording ) { fuse_emulation_unpause(); return; }
 
   rzx_start_recording( recording, 1 );
@@ -459,9 +373,9 @@ gtkui_rzx_start( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
   fuse_emulation_unpause();
 }
 
-/* Called when `File/Recording/Record from snapshot' selected */
-static void
-gtkui_rzx_start_snap( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+void
+menu_file_recording_recordfromsnapshot( GtkWidget *widget GCC_UNUSED,
+					gpointer data GCC_UNUSED )
 {
   char *snap, *recording;
 
@@ -469,10 +383,10 @@ gtkui_rzx_start_snap( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 
   fuse_emulation_pause();
 
-  snap = gtkui_fileselector_get_filename( "Fuse - Load Snapshot " );
+  snap = menu_get_filename( "Fuse - Load Snapshot " );
   if( !snap ) { fuse_emulation_unpause(); return; }
 
-  recording = gtkui_fileselector_get_filename( "Fuse - Start Recording" );
+  recording = menu_get_filename( "Fuse - Start Recording" );
   if( !recording ) { free( snap ); fuse_emulation_unpause(); return; }
 
   if( snapshot_read( snap ) ) {
@@ -488,41 +402,9 @@ gtkui_rzx_start_snap( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
   fuse_emulation_unpause();
 }
 
-/* Called when File/Recording/Stop selected */
-static void
-gtkui_rzx_stop( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
-  if( rzx_recording ) rzx_stop_recording();
-  if( rzx_playback  ) rzx_stop_playback( 1 );
-}
-
-/* Called when File/Recording/Play selected */
-static void
-gtkui_rzx_play( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
-  char *recording;
-
-  if( rzx_playback || rzx_recording ) return;
-
-  fuse_emulation_pause();
-
-  recording = gtkui_fileselector_get_filename( "Fuse - Start Replay" );
-  if( !recording ) { fuse_emulation_unpause(); return; }
-
-  rzx_start_playback( recording, gtkui_open_snap );
-
-  free( recording );
-
-  display_refresh_all();
-
-  ui_menu_activate( UI_MENU_ITEM_RECORDING, 1 );
-
-  fuse_emulation_unpause();
-}
-
-/* Called when File/AY Logging/Record selected */
-static void
-gtkui_psg_start( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+void
+menu_file_aylogging_record( GtkWidget *widget GCC_UNUSED,
+			    gpointer data GCC_UNUSED )
 {
   char *psgfile;
 
@@ -530,7 +412,7 @@ gtkui_psg_start( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 
   fuse_emulation_pause();
 
-  psgfile = gtkui_fileselector_get_filename( "Fuse - Start AY log" );
+  psgfile = menu_get_filename( "Fuse - Start AY log" );
   if ( !psgfile ) { fuse_emulation_unpause(); return; }
 
   psg_start_recording( psgfile );
@@ -544,61 +426,15 @@ gtkui_psg_start( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
   fuse_emulation_unpause();
 }
 
-/* Called when File/AY Logging/Stop selected */
-static void
-gtkui_psg_stop( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
-  if ( !psg_recording ) return;
-  psg_stop_recording();
-
-  ui_menu_activate( UI_MENU_ITEM_AY_LOGGING, 0 );
-
-  return;
-}
-
-static int
-gtkui_open_snap( void )
-{
-  char *filename;
-  int error;
-
-  filename = gtkui_fileselector_get_filename( "Fuse - Load Snapshot" );
-  if( !filename ) return -1;
-
-  error = snapshot_read( filename );
-  free( filename );
-  return error;
-}
-
-/* File/Open SCR Screenshot */
-static void
-gtkui_open_scr( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+void
+menu_file_savescreenasscr( GtkWidget *widget GCC_UNUSED,
+			   gpointer data GCC_UNUSED )
 {
   char *filename;
 
   fuse_emulation_pause();
 
-  filename =
-    gtkui_fileselector_get_filename( "Fuse - Open SCR Screenshot" );
-  if( !filename ) { fuse_emulation_unpause(); return; }
-
-  screenshot_scr_read( filename );
-
-  free( filename );
-
-  fuse_emulation_unpause();
-}
-
-/* File/Save Screenshot as SCR */
-static void
-gtkui_save_scr( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
-  char *filename;
-
-  fuse_emulation_pause();
-
-  filename =
-    gtkui_fileselector_get_filename( "Fuse - Save Screenshot as SCR" );
+  filename = menu_get_filename( "Fuse - Save Screenshot as SCR" );
   if( !filename ) { fuse_emulation_unpause(); return; }
 
   screenshot_scr_write( filename );
@@ -609,23 +445,23 @@ gtkui_save_scr( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 }
 
 #ifdef USE_LIBPNG
-/* File/Save Screenshot as PNG */
-static void
-gtkui_save_screen( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+void
+menu_file_savescreenaspng( GtkWidget *widget GCC_UNUSED,
+			   gpointer data GCC_UNUSED )
 {
   scaler_type scaler;
   char *filename;
 
   fuse_emulation_pause();
 
-  scaler = ui_get_scaler( screenshot_available_scalers );
+  scaler = menu_get_scaler( screenshot_available_scalers );
   if( scaler == SCALER_NUM ) {
     fuse_emulation_unpause();
     return;
   }
 
   filename =
-    gtkui_fileselector_get_filename( "Fuse - Save Screenshot as PNG" );
+    menu_get_filename( "Fuse - Save Screenshot as PNG" );
   if( !filename ) { fuse_emulation_unpause(); return; }
 
   screenshot_save();
@@ -638,8 +474,8 @@ gtkui_save_screen( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 #endif				/* #ifdef USE_LIBPNG */
 
 /* Called by the menu when File/Exit selected */
-static void
-gtkui_quit( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+void
+menu_file_exit( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 {
   int error;
 
@@ -651,31 +487,14 @@ gtkui_quit( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 
     /* Stop the paused state to allow us to exit (occurs from main
        emulation loop) */
-    if( paused ) gtkui_pause( NULL, NULL );
+    if( paused ) menu_machine_pause( NULL, NULL );
   }
 }
 
-/* Called by the menu when Options/Filter selected */
-static void
-select_filter( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
-  scaler_type scaler;
-
-  /* Stop emulation */
-  fuse_emulation_pause();
-
-  scaler = ui_get_scaler( scaler_is_supported );
-  if( scaler != SCALER_NUM && scaler != current_scaler )
-    scaler_select_scaler( scaler );
-
-  /* Carry on with emulation again */
-  fuse_emulation_unpause();
-}
-
 /* Select a graphics filter from those for which `available' returns
    true */
 scaler_type
-ui_get_scaler( ui_scaler_available available )
+menu_get_scaler( scaler_available_fn selector )
 {
   gtkui_select_info dialog;
   GtkAccelGroup *accel_group;
@@ -688,7 +507,7 @@ ui_get_scaler( ui_scaler_available available )
 
   /* Store the function which tells us which scalers are currently
      available */
-  dialog.available = available;
+  dialog.selector = selector;
 
   /* No scaler currently selected */
   dialog.selected = SCALER_NUM;
@@ -708,7 +527,7 @@ ui_get_scaler( ui_scaler_available available )
 
   for( scaler = 0; scaler < SCALER_NUM; scaler++ ) {
 
-    if( !available( scaler ) ) continue;
+    if( !selector( scaler ) ) continue;
 
     dialog.buttons[ count ] =
       gtk_radio_button_new_with_label( button_group, scaler_name( scaler ) );
@@ -735,7 +554,7 @@ ui_get_scaler( ui_scaler_available available )
 
   /* Add the necessary callbacks */
   gtk_signal_connect( GTK_OBJECT( ok_button ), "clicked",
-		      GTK_SIGNAL_FUNC( select_filter_done ),
+		      GTK_SIGNAL_FUNC( menu_options_filter_done ),
 		      (gpointer) &dialog );
   gtk_signal_connect_object( GTK_OBJECT( cancel_button ), "clicked",
 			     GTK_SIGNAL_FUNC( gtkui_destroy_widget_and_quit ),
@@ -764,7 +583,7 @@ ui_get_scaler( ui_scaler_available available )
 
 /* Callback used by the filter selection dialog */
 static void
-select_filter_done( GtkWidget *widget GCC_UNUSED, gpointer user_data )
+menu_options_filter_done( GtkWidget *widget GCC_UNUSED, gpointer user_data )
 {
   int i, count;
   gtkui_select_info *ptr = (gtkui_select_info*)user_data;
@@ -773,7 +592,7 @@ select_filter_done( GtkWidget *widget GCC_UNUSED, gpointer user_data )
 
   for( i = 0; i < SCALER_NUM; i++ ) {
 
-    if( !ptr->available( i ) ) continue;
+    if( !ptr->selector( i ) ) continue;
 
     if( gtk_toggle_button_get_active(
 	  GTK_TOGGLE_BUTTON( ptr->buttons[ count ] )
@@ -789,18 +608,9 @@ select_filter_done( GtkWidget *widget GCC_UNUSED, gpointer user_data )
   gtk_main_quit();
 }
 
-#ifdef HAVE_LIB_XML2
-/* Options/Save */
-static void
-save_options( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
-  settings_write_config( &settings_current );
-}
-#endif				/* #ifdef HAVE_LIB_XML2 */
-
 /* Machine/Pause */
-static void
-gtkui_pause( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+void
+menu_machine_pause( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 {
   int error;
 
@@ -827,8 +637,8 @@ gtkui_pause( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 }
 
 /* Called by the menu when Machine/Reset selected */
-static void
-gtkui_reset( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+void
+menu_machine_reset( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 {
   if( gtkui_confirm( "Reset?" ) && machine_reset() ) {
     ui_error( UI_ERROR_ERROR, "couldn't reset machine: giving up!" );
@@ -839,8 +649,8 @@ gtkui_reset( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 }
 
 /* Called by the menu when Machine/Select selected */
-static void
-gtkui_select( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+void
+menu_machine_select( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 {
   gtkui_select_info dialog;
   GtkAccelGroup *accel_group;
@@ -894,7 +704,7 @@ gtkui_select( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 
   /* Add the necessary callbacks */
   gtk_signal_connect( GTK_OBJECT( ok_button ), "clicked",
-		      GTK_SIGNAL_FUNC( gtkui_select_done ),
+		      GTK_SIGNAL_FUNC( menu_machine_select_done ),
 		      (gpointer) &dialog );
   gtk_signal_connect_object( GTK_OBJECT( cancel_button ), "clicked",
 			     GTK_SIGNAL_FUNC( gtkui_destroy_widget_and_quit ),
@@ -924,10 +734,10 @@ gtkui_select( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 
 /* Callback used by the machine selection dialog */
 static void
-gtkui_select_done( GtkWidget *widget GCC_UNUSED, gpointer user_data )
+menu_machine_select_done( GtkWidget *widget GCC_UNUSED, gpointer user_data )
 {
   int i;
-  gtkui_select_info *ptr = (gtkui_select_info*)user_data;
+  gtkui_select_info *ptr = user_data;
 
   for( i=0; i<machine_count; i++ ) {
     if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(ptr->buttons[i]) ) &&
@@ -942,68 +752,13 @@ gtkui_select_done( GtkWidget *widget GCC_UNUSED, gpointer user_data )
   gtk_main_quit();
 }
 
-/* Machine/Break */
-static void
-gtkui_break( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+void
+menu_machine_debugger( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 {
   debugger_mode = DEBUGGER_MODE_HALTED;
   if( paused ) ui_debugger_activate();
 }
 
-/* Machine/NMI */
-static void
-gtkui_nmi( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
-  /* Add an NMI to go off as soon as possible */
-  if( event_add( 0, EVENT_TYPE_NMI ) ) return;
-}
-    
-/* Called by the menu when Tape/Open selected */
-static void
-gtkui_tape_open( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
-  char *filename;
-
-  fuse_emulation_pause();
-
-  filename = gtkui_fileselector_get_filename( "Fuse - Open Tape" );
-  if( !filename ) { fuse_emulation_unpause(); return; }
-
-  tape_open_default_autoload( filename );
-
-  free( filename );
-
-  fuse_emulation_unpause();
-}
-
-/* Called by the menu when Tape/Play selected */
-static void
-gtkui_tape_play( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
-  tape_toggle_play();
-}
-
-/* Called by the menu when Tape/Rewind selected */
-static void
-gtkui_tape_rewind( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
-  tape_select_block( 0 );
-}
-
-/* Called by the menu when Tape/Clear selected */
-static void
-gtkui_tape_clear( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
-  tape_close();
-}
-
-/* Called by the menu when Tape/Write selected */
-static void
-gtkui_tape_write( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
-  ui_tape_write();
-}
-
 int
 ui_tape_write( void )
 {
@@ -1011,7 +766,7 @@ ui_tape_write( void )
 
   fuse_emulation_pause();
 
-  filename = gtkui_fileselector_get_filename( "Fuse - Write Tape" );
+  filename = menu_get_filename( "Fuse - Write Tape" );
   if( !filename ) { fuse_emulation_unpause(); return 1; }
 
   tape_write( filename );
@@ -1023,61 +778,6 @@ ui_tape_write( void )
   return 0;
 }
 
-/* Called by the menu when Disk/Drive ?:/Open selected */
-static void
-disk_open( gpointer data GCC_UNUSED, guint action,
-	   GtkWidget *widget GCC_UNUSED )
-{
-  guint which;
-
-  which = action - 1;
-
-  gtkui_disk_open( which );
-}
-
-static void gtkui_disk_open( specplus3_drive_number drive )
-{
-  char *filename;
-
-  fuse_emulation_pause();
-
-  filename = gtkui_fileselector_get_filename(
-    ( drive == SPECPLUS3_DRIVE_A ? "Fuse - Insert disk into drive A:" :
-                                   "Fuse - Insert disk into drive B:" ) );
-  if( !filename ) { fuse_emulation_unpause(); return; }
-
-#ifdef HAVE_765_H
-  if( machine_current->machine == LIBSPECTRUM_MACHINE_PLUS3 ) {
-    specplus3_disk_insert( drive, filename );
-  } else
-#endif				/* #ifdef HAVE_765_H */
-    trdos_disk_insert( drive, filename );
-
-  free( filename );
-
-  fuse_emulation_unpause();
-}
-
-static void
-disk_eject( gpointer data GCC_UNUSED, guint action,
-	    GtkWidget *widget GCC_UNUSED )
-{
-  guint parameter, which, write;
-
-  parameter = action - 1;
-  which = parameter & 0x01;
-  write = parameter & 0x02;
-
-#ifdef HAVE_765_H
-  if( machine_current->machine == LIBSPECTRUM_MACHINE_PLUS3 ) {
-    specplus3_disk_eject( which, write );
-    return;
-  }
-#endif				/* #ifdef HAVE_765_H */
-
-  trdos_disk_eject( which, write );
-}
-
 #ifdef HAVE_765_H
 int
 ui_plus3_disk_write( specplus3_drive_number which )
@@ -1087,7 +787,7 @@ ui_plus3_disk_write( specplus3_drive_number which )
   drive = which == SPECPLUS3_DRIVE_A ? 'A' : 'B';
 
   if( !specplus3_disk_present( which ) ) {
-    ui_error( UI_ERROR_INFO, "No disk present in drive %c:", drive );
+    ui_error( UI_ERROR_WARNING, "No disk present in drive %c:", drive );
     return 0;
   }
 
@@ -1095,7 +795,7 @@ ui_plus3_disk_write( specplus3_drive_number which )
 
   snprintf( title, 80, "Fuse - Write +3 Disk %c:", drive );
 
-  filename = gtkui_fileselector_get_filename( title );
+  filename = menu_get_filename( title );
   if( !filename ) { fuse_emulation_unpause(); return 1; }
 
   specplus3_disk_write( which, filename );
@@ -1119,7 +819,7 @@ ui_trdos_disk_write( trdos_drive_number which )
 
   snprintf( title, 80, "Fuse - Write TR-DOS Disk %c:", drive );
 
-  filename = gtkui_fileselector_get_filename( title );
+  filename = menu_get_filename( title );
   if( !filename ) { fuse_emulation_unpause(); return 1; }
 
   trdos_disk_write( which, filename );
@@ -1131,33 +831,8 @@ ui_trdos_disk_write( trdos_drive_number which )
   return 0;
 }
 
-/* Cartridge/Insert */
-static void
-cartridge_insert( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
-  char *filename;
-
-  fuse_emulation_pause();
-
-  filename = gtkui_fileselector_get_filename( "Fuse - Insert Cartridge" );
-  if( !filename ) { fuse_emulation_unpause(); return; }
-
-  dck_insert( filename );
-
-  free( filename );
-
-  fuse_emulation_unpause();
-}
-
-/* Cartridge/Eject */
-static void
-cartridge_eject( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
-{
-  dck_eject();
-}
-
-static void
-gtkui_help_keyboard( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+void
+menu_help_keyboard( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 {
   gtkui_picture( "keyboard.scr", 0 );
 }
@@ -1172,8 +847,6 @@ gtkui_destroy_widget_and_quit( GtkWidget *widget, gpointer data GCC_UNUSED )
 
 /* Functions to activate and deactivate certain menu items */
 
-
-
 static int
 set_menu_item_active( const char *path, int active )
 {
@@ -1198,58 +871,101 @@ set_menu_item_active( const char *path, int active )
   return 0;
 }
 
-int ui_menu_activate( ui_menu_item item, int active )
+struct menu_item_entries {
+  ui_menu_item item;
+  const char *string1;
+  const char *string2; int string2_inverted;
+  const char *string3; int string3_inverted;
+  const char *string4; int string4_inverted;
+};
+
+struct menu_item_entries menu_item_lookup[] = {
+  
+  { UI_MENU_ITEM_MEDIA_CARTRIDGE_DOCK, "/Media/Cartridge/Timex Dock" },
+
+  { UI_MENU_ITEM_MEDIA_CARTRIDGE_DOCK_EJECT,
+    "/Media/Cartridge/Timex Dock/Eject" },
+
+  { UI_MENU_ITEM_MEDIA_CARTRIDGE_IF2, "/Media/Cartridge/Interface II" },
+
+  { UI_MENU_ITEM_MEDIA_CARTRIDGE_IF2_EJECT,
+    "/Media/Cartridge/Interface II/Eject" },
+
+  { UI_MENU_ITEM_MEDIA_DISK, "/Media/Disk" },
+
+  { UI_MENU_ITEM_MEDIA_DISK_A_EJECT,
+    "/Media/Disk/Drive A:/Eject",
+    "/Media/Disk/Drive A:/Eject and write...", 0 },
+
+  { UI_MENU_ITEM_MEDIA_DISK_B_EJECT,
+    "/Media/Disk/Drive B:/Eject",
+    "/Media/Disk/Drive B:/Eject and write...", 0 },
+
+  { UI_MENU_ITEM_MEDIA_IDE, "/Media/IDE" },
+
+  { UI_MENU_ITEM_MEDIA_IDE_SIMPLE8BIT, "/Media/IDE/Simple 8-bit" },
+
+  { UI_MENU_ITEM_MEDIA_IDE_SIMPLE8BIT_MASTER_EJECT,
+    "/Media/IDE/Simple 8-bit/Master/Commit",
+    "/Media/IDE/Simple 8-bit/Master/Eject", 0 },
+
+  { UI_MENU_ITEM_MEDIA_IDE_SIMPLE8BIT_SLAVE_EJECT,
+    "/Media/IDE/Simple 8-bit/Slave/Commit",
+    "/Media/IDE/Simple 8-bit/Slave/Eject", 0 },
+
+  { UI_MENU_ITEM_MEDIA_IDE_ZXATASP, "/Media/IDE/ZXATASP" },
+
+  { UI_MENU_ITEM_MEDIA_IDE_ZXATASP_MASTER_EJECT,
+    "/Media/IDE/ZXATASP/Master/Commit",
+    "/Media/IDE/ZXATASP/Master/Eject", 0 },
+
+  { UI_MENU_ITEM_MEDIA_IDE_ZXATASP_SLAVE_EJECT,
+    "/Media/IDE/ZXATASP/Slave/Commit",
+    "/Media/IDE/ZXATASP/Slave/Eject", 0 },
+
+  { UI_MENU_ITEM_MEDIA_IDE_ZXCF, "/Media/IDE/ZXCF CompactFlash" },
+
+  { UI_MENU_ITEM_MEDIA_IDE_ZXCF_EJECT,
+    "/Media/IDE/ZXCF CompactFlash/Commit",
+    "/Media/IDE/ZXCF CompactFlash/Eject", 0 },
+
+  { UI_MENU_ITEM_RECORDING,
+    "/File/Recording/Stop", 
+    "/File/Recording/Record...", 1,
+    "/File/Recording/Record from snapshot...", 1,
+    "/File/Recording/Play...", 1 },
+
+  { UI_MENU_ITEM_AY_LOGGING,
+    "/File/AY Logging/Stop",
+    "/File/AY Logging/Record...", 1, },
+
+  { UI_MENU_ITEM_AY_LOGGING, NULL },	/* End marker */
+};
+
+int
+ui_menu_activate( ui_menu_item item, int active )
 {
-  int error;
+  struct menu_item_entries *ptr;
 
-  switch( item ) {
+  for( ptr = menu_item_lookup; ptr->string1; ptr++ ) {
 
-  case UI_MENU_ITEM_MEDIA_CARTRIDGE:
-    return set_menu_item_active( "/Media/Cartridge", active );
-
-  case UI_MENU_ITEM_MEDIA_CARTRIDGE_EJECT:
-    return set_menu_item_active( "/Media/Cartridge/Eject", active );
-
-  case UI_MENU_ITEM_MEDIA_DISK:
-    return set_menu_item_active( "/Media/Disk", active );
-
-  case UI_MENU_ITEM_MEDIA_DISK_A_EJECT:
-    error = set_menu_item_active( "/Media/Disk/Drive A:/Eject", active );
-    if( error ) return error;
-
-    return set_menu_item_active( "/Media/Disk/Drive A:/Eject and write...",
-				 active );
-
-  case UI_MENU_ITEM_MEDIA_DISK_B_EJECT:
-    error = set_menu_item_active( "/Media/Disk/Drive B:/Eject", active );
-    if( error ) return error;
-
-    return set_menu_item_active( "/Media/Disk/Drive B:/Eject and write...",
-				 active );
-
-  case UI_MENU_ITEM_RECORDING:
-    error = set_menu_item_active( "/File/Recording/Record...", !active );
-    if( error ) return error;
-
-    error = set_menu_item_active( "/File/Recording/Record from snapshot...",
-				  !active );
-    if( error ) return error;
-
-    error = set_menu_item_active( "/File/Recording/Play...", !active );
-    if( error ) return error;
-
-    return set_menu_item_active( "/File/Recording/Stop", active );
-
-  case UI_MENU_ITEM_AY_LOGGING:
-    error = set_menu_item_active( "/File/AY Logging/Record...", !active );
-    if( error ) return error;
-
-    return set_menu_item_active( "/File/AY Logging/Stop", active );
+    if( item == ptr->item ) {
+      set_menu_item_active( ptr->string1, active );
+      if( ptr->string2 ) 
+	set_menu_item_active( ptr->string2,
+			      ptr->string2_inverted ? !active : active );
+      if( ptr->string3 ) 
+	set_menu_item_active( ptr->string3,
+			      ptr->string3_inverted ? !active : active );
+      if( ptr->string4 )
+	set_menu_item_active( ptr->string4,
+			      ptr->string4_inverted ? !active : active );
+      return 0;
+    }
 
   }
 
-  ui_error( UI_ERROR_ERROR, "Attempt to activate Unknown menu item %d",
-	    item );
+  ui_error( UI_ERROR_ERROR, "ui_menu_activate: unknown item %d\n", item );
   return 1;
 }
 
