@@ -1,7 +1,7 @@
 /* roms.c: ROM selector dialog box
-   Copyright (c) 2003 Philip Kendall
+   Copyright (c) 2003-2004 Philip Kendall
 
-   $Id: roms.c,v 1.8 2003/12/02 14:56:33 pak21 Exp $
+   $Id: roms.c,v 1.16 2004/06/17 09:17:04 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 
 #ifdef UI_GTK		/* Use this file iff we're using GTK+ */
 
+#include <stdio.h>
 #include <stdlib.h>
 
 #include <gdk/gdkkeysyms.h>
@@ -36,23 +37,34 @@
 #include "compat.h"
 #include "fuse.h"
 #include "gtkinternals.h"
+#include "menu.h"
 #include "settings.h"
+#include "ui/ui.h"
 
-static void add_rom( GtkWidget *table, gint row, const char *name );
+static void add_rom( GtkBox *parent, size_t start, gint row );
 static void select_new_rom( GtkWidget *widget, gpointer data );
-static void roms_done( GtkWidget *widget, gpointer data );
+static void roms_done( GtkButton *button, gpointer data );
 
 /* The labels used to display the current ROMs */
 GtkWidget *rom[ SETTINGS_ROM_COUNT ];
 
-void
-gtkui_roms( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
+struct callback_info {
+
+  size_t start, n;
+
+};
+
+int
+menu_select_roms( libspectrum_machine machine, size_t start, size_t n )
 {
   GtkWidget *dialog;
   GtkAccelGroup *accel_group;
-  GtkWidget *table;
   GtkWidget *ok_button, *cancel_button;
+  GtkBox *vbox;
 
+  struct callback_info info;
+
+  char buffer[ 256 ];
   size_t i;
 
   /* Firstly, stop emulation */
@@ -60,16 +72,12 @@ gtkui_roms( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
 
   /* Give me a new dialog box */
   dialog = gtk_dialog_new();
-  gtk_window_set_title( GTK_WINDOW( dialog ), "Fuse - Select ROMs" );
+  snprintf( buffer, 256, "Fuse - Select ROMs - %s",
+	    libspectrum_machine_name( machine ) );
+  gtk_window_set_title( GTK_WINDOW( dialog ), buffer );
 
-  /* A table to put all the labels in */
-  table = gtk_table_new( SETTINGS_ROM_COUNT, 3, FALSE );
-  gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog )->vbox ),
-		     table );
-
-  /* And the current values of each of the ROMs */
-  for( i = 0; i < SETTINGS_ROM_COUNT; i++ )
-    add_rom( table, i, settings_rom_name[i] );
+  gtk_signal_connect( GTK_OBJECT( dialog ), "delete_event",
+		      GTK_SIGNAL_FUNC( gtkui_destroy_widget_and_quit ), NULL );
 
   /* Create the OK and Cancel buttons */
   ok_button = gtk_button_new_with_label( "OK" );
@@ -80,21 +88,30 @@ gtkui_roms( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
   gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog )->action_area ),
 		     cancel_button );
   
+  info.start = start;
+  info.n = n;
+  gtk_signal_connect( GTK_OBJECT( ok_button ), "clicked",
+		      GTK_SIGNAL_FUNC( roms_done ), &info );
+
   gtk_signal_connect_object( GTK_OBJECT( ok_button ), "clicked",
-			     GTK_SIGNAL_FUNC( roms_done ),
+			     GTK_SIGNAL_FUNC( gtkui_destroy_widget_and_quit ),
 			     GTK_OBJECT( dialog ) );
+
   gtk_signal_connect_object( GTK_OBJECT( cancel_button ), "clicked",
 			     GTK_SIGNAL_FUNC( gtkui_destroy_widget_and_quit ),
 			     GTK_OBJECT( dialog ) );
-  gtk_signal_connect( GTK_OBJECT( dialog ), "delete_event",
-		      GTK_SIGNAL_FUNC( gtkui_destroy_widget_and_quit ), NULL );
 
+  /* And the current values of each of the ROMs */
+  vbox = GTK_BOX( GTK_DIALOG( dialog )->vbox );
+
+  gtk_container_set_border_width( GTK_CONTAINER( vbox ), 5 );
+  for( i = 0; i < n; i++ ) add_rom( vbox, start, i );
+
+  /* Keyboard accelerator (Esc to cancel) */
   accel_group = gtk_accel_group_new();
   gtk_window_add_accel_group( GTK_WINDOW( dialog ), accel_group );
-
-  /* Allow Esc to cancel */
-  gtk_widget_add_accelerator( cancel_button, "clicked",
-			      accel_group, GDK_Escape, 0, 0);
+  gtk_widget_add_accelerator( cancel_button, "clicked", accel_group,
+			      GDK_Escape, 0, 0 );
 
   /* Users shouldn't be able to resize this window */
   gtk_window_set_policy( GTK_WINDOW( dialog ), FALSE, FALSE, TRUE );
@@ -109,29 +126,33 @@ gtkui_roms( GtkWidget *widget GCC_UNUSED, gpointer data GCC_UNUSED )
   /* And then carry on with emulation again */
   fuse_emulation_unpause();
 
-  return;
+  return 0;
 }
 
 static void
-add_rom( GtkWidget *table, gint row, const char *name )
+add_rom( GtkBox *parent, size_t start, gint row )
 {
-  GtkWidget *label, *change_button;
+  GtkWidget *frame, *hbox, *change_button;
+  char buffer[ 80 ], **setting;
 
-  label = gtk_label_new( name );
-  gtk_table_attach( GTK_TABLE( table ), label, 0, 1, row, row + 1,
-		    0, 0, 2, 2 );
+  snprintf( buffer, 80, "ROM %d", row );
+  frame = gtk_frame_new( buffer );
+  gtk_box_pack_start( parent, frame, FALSE, FALSE, 2 );
 
-  rom[ row ] =
-    gtk_label_new( *( settings_get_rom_setting( &settings_current, row ) ) );
-  gtk_table_attach( GTK_TABLE( table ), rom[ row ], 1, 2, row, row + 1,
-		    0, 0, 2, 2 );
+  hbox = gtk_hbox_new( FALSE, 4 );
+  gtk_container_set_border_width( GTK_CONTAINER( hbox ), 4 );
+  gtk_container_add( GTK_CONTAINER( frame ), hbox );
 
-  change_button = gtk_button_new_with_label( "Change" );
+  setting = settings_get_rom_setting( &settings_current, start + row );
+  rom[ row ] = gtk_entry_new();
+  gtk_entry_set_text( GTK_ENTRY( rom[ row ] ), *setting );
+  gtk_box_pack_start( GTK_BOX( hbox ), rom[ row ], FALSE, FALSE, 2 );
+
+  change_button = gtk_button_new_with_label( "Select..." );
   gtk_signal_connect( GTK_OBJECT( change_button ), "clicked",
 		      GTK_SIGNAL_FUNC( select_new_rom ),
 		      rom[ row ] );
-  gtk_table_attach( GTK_TABLE( table ), change_button, 2, 3, row, row + 1,
-		    0, 0, 2, 2 );
+  gtk_box_pack_start( GTK_BOX( hbox ), change_button, FALSE, FALSE, 2 );
 }
 
 static void
@@ -139,31 +160,32 @@ select_new_rom( GtkWidget *widget GCC_UNUSED, gpointer data )
 {
   char *filename;
 
-  GtkWidget *label = data;
+  GtkWidget *entry = data;
 
-  filename = gtkui_fileselector_get_filename( "Fuse - Select ROM" );
+  filename = menu_get_filename( "Fuse - Select ROM" );
   if( !filename ) return;
 
-  gtk_label_set( GTK_LABEL( label ), filename );
+  gtk_entry_set_text( GTK_ENTRY( entry ), filename );
 }
 
 static void
-roms_done( GtkWidget *widget, gpointer data GCC_UNUSED )
+roms_done( GtkButton *button GCC_UNUSED, gpointer data )
 {
   size_t i;
   int error;
   
-  char *string;
+  char **setting; const char *string;
+
+  struct callback_info *info = data;
+
+  for( i = 0; i < info->n; i++ ) {
+
+    setting = settings_get_rom_setting( &settings_current, info->start + i );
+    string = gtk_entry_get_text( GTK_ENTRY( rom[i] ) );
+
+    error = settings_set_string( setting, string ); if( error ) return;
 
-  for( i = 0; i < SETTINGS_ROM_COUNT; i++ ) {
-    gtk_label_get( GTK_LABEL( rom[i] ), &string );
-    error =
-      settings_set_string( settings_get_rom_setting( &settings_current, i ),
-			   string );
-    if( error ) return;
   }
-
-  gtkui_destroy_widget_and_quit( widget, NULL );
 }
 
 #endif			/* #ifdef UI_GTK */
