@@ -1,7 +1,7 @@
 /* scaler.c: code for selecting (etc) scalers
  * Copyright (C) 2003 Fredrick Meunier, Philip Kendall
  * 
- * $Id: scaler.c,v 1.24 2004/01/02 14:01:28 pak21 Exp $
+ * $Id: scaler.c,v 1.25 2004/07/04 18:48:38 pak21 Exp $
  *
  * Originally taken from ScummVM - Scumm Interpreter
  * Copyright (C) 2001  Ludvig Strigeus
@@ -65,7 +65,7 @@ static void expand_dotmatrix( int *x, int *y, int *w, int *h,
 
 /* Information on each of the available scalers. Make sure this array stays
    in the same order as scaler.h:scaler_type */
-static struct scaler_info available_scalers[] = {
+static const struct scaler_info available_scalers[] = {
 
   { "Timex Half (smoothed)", "half", SCALER_FLAGS_NONE,	       0.5,
     scaler_Half_16,       scaler_Half_32,       NULL                },
