@@ -1,7 +1,7 @@
 /* ui.h: General UI event handling routines
    Copyright (c) 2000-2004 Philip Kendall
 
-   $Id: ui.h,v 1.42 2004/01/08 17:02:57 pak21 Exp $
+   $Id: ui.h,v 1.51 2004/05/28 09:37:40 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,8 +32,7 @@
 #include <libspectrum.h>
 
 #include "compat.h"
-#include "keysyms.h"
-#include "specplus3.h"
+#include "machines/specplus3.h"
 #include "trdos.h"
 #include "ui/scaler/scaler.h"
 
@@ -41,18 +40,25 @@
 typedef enum ui_error_level {
 
   UI_ERROR_INFO,		/* Informational message */
+  UI_ERROR_WARNING,		/* Something is wrong, but it's not that
+				   important */
   UI_ERROR_ERROR,		/* An actual error */
 
 } ui_error_level;
 
-extern const keysyms_key_info keysyms_data[];
-
 int ui_init(int *argc, char ***argv);
 int ui_event(void);
-int ui_verror( ui_error_level severity, const char *format, va_list ap )
-     GCC_PRINTF( 2, 0 );
 int ui_end(void);
 
+/* Error handling routines */
+int ui_error( ui_error_level severity, const char *format, ... )
+     GCC_PRINTF( 2, 3 );
+libspectrum_error ui_libspectrum_error( libspectrum_error error,
+					const char *format, va_list ap );
+int ui_verror( ui_error_level severity, const char *format, va_list ap );
+int ui_error_specific( ui_error_level severity, const char *message );
+void ui_error_frame( void );
+
 /* Callbacks used by the debugger */
 int ui_debugger_activate( void );
 int ui_debugger_deactivate( int interruptable );
@@ -60,10 +66,6 @@ int ui_debugger_update( void );
 int ui_debugger_disassemble( libspectrum_word address );
 
 /* Functions defined in ../ui.c */
-int ui_error( ui_error_level severity, const char *format, ... )
-     GCC_PRINTF( 2, 3 );
-libspectrum_error ui_libspectrum_error( libspectrum_error error,
-					const char *format, va_list ap );
 
 /* Confirm whether we want to save some data before overwriting it */
 typedef enum ui_confirm_save_t {
@@ -83,19 +85,26 @@ int ui_tape_write( void );
 int ui_plus3_disk_write( specplus3_drive_number which );
 int ui_trdos_disk_write( trdos_drive_number which );
 
-/* Select a scaler from those for which `available' returns true */
-typedef int (*ui_scaler_available)( scaler_type scaler );
-scaler_type ui_get_scaler( ui_scaler_available available );
-
 /* Routines to (de)activate certain menu items */
 
 typedef enum ui_menu_item {
 
-  UI_MENU_ITEM_MEDIA_CARTRIDGE,
-  UI_MENU_ITEM_MEDIA_CARTRIDGE_EJECT,
+  UI_MENU_ITEM_MEDIA_CARTRIDGE_DOCK,
+  UI_MENU_ITEM_MEDIA_CARTRIDGE_DOCK_EJECT,
+  UI_MENU_ITEM_MEDIA_CARTRIDGE_IF2,
+  UI_MENU_ITEM_MEDIA_CARTRIDGE_IF2_EJECT,
   UI_MENU_ITEM_MEDIA_DISK,
   UI_MENU_ITEM_MEDIA_DISK_A_EJECT,
   UI_MENU_ITEM_MEDIA_DISK_B_EJECT,
+  UI_MENU_ITEM_MEDIA_IDE,
+  UI_MENU_ITEM_MEDIA_IDE_SIMPLE8BIT,
+  UI_MENU_ITEM_MEDIA_IDE_SIMPLE8BIT_MASTER_EJECT,
+  UI_MENU_ITEM_MEDIA_IDE_SIMPLE8BIT_SLAVE_EJECT,
+  UI_MENU_ITEM_MEDIA_IDE_ZXATASP,
+  UI_MENU_ITEM_MEDIA_IDE_ZXATASP_MASTER_EJECT,
+  UI_MENU_ITEM_MEDIA_IDE_ZXATASP_SLAVE_EJECT,
+  UI_MENU_ITEM_MEDIA_IDE_ZXCF,
+  UI_MENU_ITEM_MEDIA_IDE_ZXCF_EJECT,
   UI_MENU_ITEM_RECORDING,
   UI_MENU_ITEM_AY_LOGGING,
 
