@@ -1,7 +1,7 @@
 /* win32display.h:  Routines for dealing with the Win32 DirectDraw display
    Copyright (c) 2003 Marek Januszewski
 
-   $Id: win32display.h,v 1.1 2003/12/31 16:11:23 pak21 Exp $
+   $Id: win32display.h,v 1.2 2004/07/11 23:07:21 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,24 +28,7 @@
 
 #ifdef UI_WIN32			/* Use this iff we're using UI_WIN32 */
 
-#include <windows.h>
-#include <ddraw.h>
-
-/* window handler */
-HWND hWnd;
-
-/* windows message */
-MSG msg;
-
-/* DirectDraw objects */
-LPDIRECTDRAW lpdd;
-DDSURFACEDESC ddsd;
-LPDIRECTDRAWSURFACE lpdds, /* primary surface */
-                    lpdds2; /* off screen surface */
-LPDIRECTDRAWCLIPPER lpddc; /* clipper for the window */
-
-/* context */
-HDC dc;
+#include "win32internals.h"
 
 void blit( void );
 
