@@ -1,7 +1,7 @@
 /* widget_internals.h: Functions internal to the widget code
-   Copyright (c) 2001-2003 Matan Ziv-Av, Philip Kendall
+   Copyright (c) 2001-2004 Matan Ziv-Av, Philip Kendall
 
-   $Id: widget_internals.h,v 1.63 2004/01/05 23:32:41 pak21 Exp $
+   $Id: widget_internals.h,v 1.70 2004/04/11 10:08:25 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -88,18 +88,17 @@ extern size_t widget_numfiles;
 
 int widget_filesel_draw( void* data );
 int widget_filesel_finish( widget_finish_state finished );
-void widget_filesel_keyhandler( keyboard_key_name key,
-				keyboard_key_name key2 );
+void widget_filesel_keyhandler( input_key key );
 
 /* Tape menu */
 
 int widget_tape_draw( void* data );
-void widget_tape_keyhandler( keyboard_key_name key, keyboard_key_name key2 );
+void widget_tape_keyhandler( input_key key );
 
 /* File menu */
 
 int widget_file_draw( void* data );
-void widget_file_keyhandler( keyboard_key_name key, keyboard_key_name key2 );
+void widget_file_keyhandler( input_key key );
 
 /* Options menu */
 int widget_menu_filter( void *data );
@@ -107,8 +106,7 @@ int widget_menu_filter( void *data );
 /* Machine menu */
 
 int widget_machine_draw( void* data );
-void widget_machine_keyhandler( keyboard_key_name key,
-				keyboard_key_name key2 );
+void widget_machine_keyhandler( input_key key );
 
 /* Keyboard picture */
 
@@ -119,82 +117,42 @@ typedef struct widget_picture_data {
 } widget_picture_data;
 
 int widget_picture_draw( void* data );
-void widget_picture_keyhandler( keyboard_key_name key,
-				keyboard_key_name key2 );
+void widget_picture_keyhandler( input_key key );
 
 /* Help menu */
 
 int widget_help_draw( void* data );
-void widget_help_keyhandler( keyboard_key_name key, keyboard_key_name key2 );
+void widget_help_keyhandler( input_key key );
 
 /* General menu code */
 
 int widget_menu_draw( void* data );
-void widget_menu_keyhandler( keyboard_key_name key, keyboard_key_name key2 );
-
-/* General callbacks */
-
-/* The callback function to call another widget */
-int widget_menu_widget( void *data );
-
-/* The data type passed to widget_menu_widget */
-typedef struct widget_menu_widget_t {
-  widget_type widget;	/* The widget to call */
-  void *data;		/* with this data parameter */
-} widget_menu_widget_t;
+void widget_menu_keyhandler( input_key key );
 
 /* More callbacks */
-int widget_menu_open( const char *filename ); /* File/Open */
-int widget_menu_save_snapshot( void *data ); /* File/Save Snapshot */
-int widget_menu_rzx_recording( void *data ); /* File/Recording/Record */
-int widget_menu_rzx_recording_snap( void *data ); /* File/Recording/Record
-						     from snap */
-int widget_menu_rzx_playback( void *data );  /* File/Recording/Play */
-int widget_menu_rzx_stop( void *data );	     /* File/Recording/Stop */
-int widget_menu_psg_record( void *data );    /* File/AY Logging/Record */
-int widget_menu_psg_stop( void *data );	     /* File/AY Logging/Stop */
-int widget_menu_save_screen( void *data );   /* File/Save Screenshot */
-int widget_menu_save_scr( void *data );	     /* File/Save Scr */
-int widget_menu_exit( void *data );	     /* File/Exit */
+scaler_type widget_select_scaler( int (*selector)( scaler_type ) );
 
-int widget_menu_save_options( void *data );  /* Options/Save */
+/* The generalised selector widget */
 
-int widget_menu_reset( void *data );	     /* Machine/Reset */
-int widget_menu_break( void *data );	     /* Machine/Break */
-int widget_menu_nmi( void *data );	     /* Machine/NMI */
+typedef struct widget_select_t {
 
-int widget_menu_play_tape( void *data );     /* Tape/Play */
-int widget_menu_rewind_tape( void *data );   /* Tape/Rewind */
-int widget_menu_clear_tape( void *data );    /* Tape/Close */
-int widget_menu_write_tape( void *data );    /* Tape/Write */
+  const char *title;	/* Dialog title */
+  const char **options;	/* The available options */
+  size_t count;		/* The number of options */
+  size_t current;	/* Which option starts active? */
 
-int widget_insert_disk_a( const char *filename ); /* Disk/Drive A:/Insert */
-int widget_insert_disk_b( const char *filename ); /* Disk/Drive B:/Insert */
-int widget_menu_eject_disk( void *data );    /* Disk/Drive ?:/Eject */
-int widget_menu_eject_write_disk( void *data ); /* Disk/Drive ?:/Eject and
-						   write */
+  int result;		/* What was selected? ( -1 if dialog cancelled ) */
 
-int widget_insert_dock( const char *filename ); /* Cart/Timex Dock/Insert */
-int widget_menu_eject_dock( void *data );    /* Cart/Timex Dock/Eject */
+} widget_select_t;
 
-int widget_menu_keyboard( void *data );	     /* Help/Keyboard Picture */
-
-/* The select scaler widget */
-
-int widget_scaler_draw( void* data );
-void widget_scaler_keyhandler( keyboard_key_name key, keyboard_key_name key2 );
-int widget_scaler_finish( widget_finish_state finished );
-
-/* The select machine widget */
-
-int widget_select_draw( void* data );
-void widget_select_keyhandler( keyboard_key_name key, keyboard_key_name key2 );
+int widget_select_draw( void *data );
+void widget_select_keyhandler( input_key key );
 int widget_select_finish( widget_finish_state finished );
 
 /* The tape browser widget */
 
 int widget_browse_draw( void* data );
-void widget_browse_keyhandler( keyboard_key_name key, keyboard_key_name key2 );
+void widget_browse_keyhandler( input_key key );
 int widget_browse_finish( widget_finish_state finished );
 
 /* The text entry widget */
@@ -205,7 +163,7 @@ typedef struct widget_text_t {
 } widget_text_t;
 
 int widget_text_draw( void* data );
-void widget_text_keyhandler( keyboard_key_name key, keyboard_key_name key2 );
+void widget_text_keyhandler( input_key key );
 int widget_text_finish( widget_finish_state finished );
 
 extern char *widget_text_text;	/* The returned text */
@@ -221,17 +179,26 @@ int widget_options_finish( widget_finish_state finished );
 /* The error widget */
 
 int widget_error_draw( void *data );
-void widget_error_keyhandler( keyboard_key_name key, keyboard_key_name key2 );
+void widget_error_keyhandler( input_key key );
 
 /* The debugger widget */
 
 int widget_debugger_draw( void *data );
-void widget_debugger_keyhandler( keyboard_key_name key,
-				 keyboard_key_name key2 );
+void widget_debugger_keyhandler( input_key key );
 
 /* The ROM selector widget */
-int widget_roms_draw( void* data );
-void widget_roms_keyhandler( keyboard_key_name key, keyboard_key_name key2 );
+
+typedef struct widget_roms_info {
+
+  int initialised;
+
+  libspectrum_machine machine;
+  size_t start, count;
+
+} widget_roms_info;
+
+int widget_roms_draw( void *data );
+void widget_roms_keyhandler( input_key key );
 int widget_roms_finish( widget_finish_state finished );
 
 /* The widgets actually available */
