@@ -1,7 +1,7 @@
 /* coretest.c: Test program for Fuse's Z80 core
    Copyright (c) 2003 Philip Kendall
 
-   $Id: coretest.c,v 1.14.2.1 2004/01/14 19:59:05 pak21 Exp $
+   $Id: coretest.c,v 1.22 2004/06/09 10:55:08 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -52,16 +52,12 @@ libspectrum_byte readbyte_internal( libspectrum_word address );
 void writebyte( libspectrum_word address, libspectrum_byte b );
 void writebyte_internal( libspectrum_word address, libspectrum_byte b );
 
-static libspectrum_dword trivial_contend_port( libspectrum_word port );
-
 static int read_test_file( const char *filename,
 			   libspectrum_dword *end_tstates );
 
 static void dump_z80_state( void );
 static void dump_memory_state( void );
 
-spectrum_port_contention_function contend_port = trivial_contend_port;
-
 int
 main( int argc, char **argv )
 {
@@ -130,15 +126,22 @@ writebyte_internal( libspectrum_word address, libspectrum_byte b )
   memory[ address ] = b;
 }
 
-static libspectrum_dword
-trivial_contend_port( libspectrum_word port )
+void
+ula_contend_port( libspectrum_word port )
 {
   printf( "%5d PC %04x\n", tstates, port );
-  return 0;
+  tstates += 4;
 }
 
 void
-contend( libspectrum_word address, libspectrum_dword time )
+contend_read( libspectrum_word address, libspectrum_dword time )
+{
+  printf( "%5d MC %04x\n", tstates, address );
+  tstates += time;
+}
+
+void
+contend_write( libspectrum_word address, libspectrum_dword time )
 {
   printf( "%5d MC %04x\n", tstates, address );
   tstates += time;
@@ -325,6 +328,8 @@ enum debugger_mode_t debugger_mode;
 
 libspectrum_byte **ROM = NULL;
 memory_page memory_map[8];
+memory_page *memory_map_home[8];
+memory_page memory_map_rom[8];
 int memory_contended[8] = { 1 };
 libspectrum_byte spectrum_contention[ 80000 ] = { 0 };
 
@@ -342,9 +347,27 @@ debugger_trap( void )
   abort();
 }
 
+int
+slt_trap( libspectrum_word address, libspectrum_byte level )
+{
+  return 0;
+}
+
 int trdos_available = 0;
 int trdos_active = 0;
 
+void
+trdos_page( void )
+{
+  abort();
+}
+
+void
+trdos_unpage( void )
+{
+  abort();
+}
+
 int
 event_add( libspectrum_dword event_time, int type )
 {
@@ -355,6 +378,8 @@ event_add( libspectrum_dword event_time, int type )
 fuse_machine_info *machine_current;
 static fuse_machine_info dummy_machine;
 
+const int LIBSPECTRUM_MACHINE_CAPABILITY_EVEN_M1 = 1;
+
 settings_info settings_current;
 
 /* Initialise the dummy variables such that we're running on a clean a
@@ -364,9 +389,12 @@ init_dummies( void )
 {
   size_t i;
 
-  for( i = 0; i < 8; i++ ) memory_map[i].page = &memory[ i * 0x2000 ];
+  for( i = 0; i < 8; i++ ) {
+    memory_map[i].page = &memory[ i * MEMORY_PAGE_SIZE ];
+  }
 
   debugger_mode = DEBUGGER_MODE_INACTIVE;
+  dummy_machine.capabilities = 0;
   dummy_machine.ram.current_rom = 0;
   machine_current = &dummy_machine;
   rzx_playback = 0;
