@@ -42,15 +42,15 @@
    result, 1 is the 3rd bit of the 1st argument and 2 is the
    third bit of the 2nd argument; the tables differ for add and subtract
    operations */
-libspectrum_byte halfcarry_add_table[] =
+const libspectrum_byte halfcarry_add_table[] =
   { 0, FLAG_H, FLAG_H, FLAG_H, 0, 0, 0, FLAG_H };
-libspectrum_byte halfcarry_sub_table[] =
+const libspectrum_byte halfcarry_sub_table[] =
   { 0, 0, FLAG_H, 0, FLAG_H, 0, FLAG_H, FLAG_H };
 
 /* Similarly, overflow can be determined by looking at the 7th bits; again
    the hash into this table is r12 */
-libspectrum_byte overflow_add_table[] = { 0, 0, 0, FLAG_V, FLAG_V, 0, 0, 0 };
-libspectrum_byte overflow_sub_table[] = { 0, FLAG_V, 0, 0, 0, 0, FLAG_V, 0 };
+const libspectrum_byte overflow_add_table[] = { 0, 0, 0, FLAG_V, FLAG_V, 0, 0, 0 };
+const libspectrum_byte overflow_sub_table[] = { 0, FLAG_V, 0, 0, 0, 0, FLAG_V, 0 };
 
 /* Some more tables; initialised in z80_init_tables() */
 
@@ -128,13 +128,13 @@ z80_interrupt( void )
     R++; rzx_instructions_offset--;
 
     switch(IM) {
-      case 0: PC = 0x0038; tstates+=12; break;
-      case 1: PC = 0x0038; tstates+=13; break;
+      case 0: PC = 0x0038; tstates += 6; break;
+      case 1: PC = 0x0038; tstates += 7; break;
       case 2: 
 	{
 	  libspectrum_word inttemp=(0x100*I)+0xff;
 	  PCL = readbyte(inttemp++); PCH = readbyte(inttemp);
-	  tstates+=19;
+	  tstates += 7;
 	  break;
 	}
       default:
@@ -167,3 +167,59 @@ z80_nmi( void )
   tstates += 11; PC = 0x0066;
 }
 
+/* Routines for transferring the Z80 contents to and from snapshots */
+int
+z80_from_snapshot( libspectrum_snap *snap )
+{
+  A  = libspectrum_snap_a ( snap ); F  = libspectrum_snap_f ( snap );
+  A_ = libspectrum_snap_a_( snap ); F_ = libspectrum_snap_f_( snap );
+
+  BC  = libspectrum_snap_bc ( snap ); DE  = libspectrum_snap_de ( snap );
+  HL  = libspectrum_snap_hl ( snap ); BC_ = libspectrum_snap_bc_( snap );
+  DE_ = libspectrum_snap_de_( snap ); HL_ = libspectrum_snap_hl_( snap );
+
+  IX = libspectrum_snap_ix( snap ); IY = libspectrum_snap_iy( snap );
+  I  = libspectrum_snap_i ( snap ); R = R7 = libspectrum_snap_r( snap );
+  SP = libspectrum_snap_sp( snap ); PC = libspectrum_snap_pc( snap );
+
+  IFF1 = libspectrum_snap_iff1( snap ); IFF2 = libspectrum_snap_iff2( snap );
+  IM = libspectrum_snap_im( snap );
+
+  z80.halted = libspectrum_snap_halted( snap );
+
+  z80.interrupts_enabled_at =
+    libspectrum_snap_last_instruction_ei( snap ) ? tstates : -1;
+
+  return 0;
+}
+  
+int
+z80_to_snapshot( libspectrum_snap *snap )
+{
+  libspectrum_byte r_register;
+
+  r_register = ( R7 & 0x80 ) | ( R & 0x7f );
+
+  libspectrum_snap_set_a  ( snap, A   ); libspectrum_snap_set_f  ( snap, F   );
+  libspectrum_snap_set_a_ ( snap, A_  ); libspectrum_snap_set_f_ ( snap, F_  );
+
+  libspectrum_snap_set_bc ( snap, BC  ); libspectrum_snap_set_de ( snap, DE  );
+  libspectrum_snap_set_hl ( snap, HL  ); libspectrum_snap_set_bc_( snap, BC_ );
+  libspectrum_snap_set_de_( snap, DE_ ); libspectrum_snap_set_hl_( snap, HL_ );
+
+  libspectrum_snap_set_ix ( snap, IX  ); libspectrum_snap_set_iy ( snap, IY  );
+  libspectrum_snap_set_i  ( snap, I   );
+  libspectrum_snap_set_r  ( snap, r_register );
+  libspectrum_snap_set_sp ( snap, SP  ); libspectrum_snap_set_pc ( snap, PC  );
+
+  libspectrum_snap_set_iff1( snap, IFF1 );
+  libspectrum_snap_set_iff2( snap, IFF2 );
+  libspectrum_snap_set_im( snap, IM );
+
+  libspectrum_snap_set_halted( snap, z80.halted );
+  libspectrum_snap_set_last_instruction_ei(
+    snap, z80.interrupts_enabled_at == tstates
+  );
+
+  return 0;
+}
