@@ -1,7 +1,7 @@
 #!/usr/bin/perl -w
 
 # z80.pl: generate C code for Z80 opcodes
-# $Id: z80.pl,v 1.54 2004/01/09 15:14:47 pak21 Exp $
+# $Id: z80.pl,v 1.64 2004/07/07 09:27:58 pak21 Exp $
 
 # Copyright (c) 1999-2003 Philip Kendall
 
@@ -58,10 +58,12 @@ sub arithmetic_logical ($$$) {
 	    print "      $opcode($arg2);\n";
 	} elsif( $arg2 eq '(REGISTER+dd)' ) {
 	    print << "CODE";
-      tstates += 5;		/* FIXME: how is this contended? */
       {
-	libspectrum_byte bytetemp = 
-	    readbyte( REGISTER + (libspectrum_signed_byte)readbyte( PC++ ) );
+	libspectrum_byte offset, bytetemp;
+	offset = readbyte( PC );
+	contend_read( PC, 1 ); contend_read( PC, 1 ); contend_read( PC, 1 );
+	contend_read( PC, 1 ); contend_read( PC, 1 ); PC++;
+	bytetemp = readbyte( REGISTER + (libspectrum_signed_byte)offset );
 	$opcode(bytetemp);
       }
 CODE
@@ -99,7 +101,7 @@ sub call_jp ($$$) {
       if( $condition_string ) {
 	$opcode();
       } else {
-	contend( PC, 3 ); contend( PC + 1, 3 ); PC += 2;
+	contend_read( PC, 3 ); contend_read( PC + 1, 3 ); PC += 2;
       }
 CALL
     }
@@ -117,8 +119,8 @@ sub cpi_cpd ($) {
 	  lookup = ( (        A & 0x08 ) >> 3 ) |
 	           ( (  (value) & 0x08 ) >> 2 ) |
 	           ( ( bytetemp & 0x08 ) >> 1 );
-	contend( HL, 1 ); contend( HL, 1 ); contend( HL, 1 ); contend( HL, 1 );
-	contend( HL, 1 );
+	contend_read( HL, 1 ); contend_read( HL, 1 ); contend_read( HL, 1 );
+	contend_read( HL, 1 ); contend_read( HL, 1 );
 	HL$modifier; BC--;
 	F = ( F & FLAG_C ) | ( BC ? ( FLAG_V | FLAG_N ) : FLAG_N ) |
 	  halfcarry_sub_table[lookup] | ( bytetemp ? 0 : FLAG_Z ) |
@@ -141,8 +143,8 @@ sub cpir_cpdr ($) {
 	  lookup = ( (        A & 0x08 ) >> 3 ) |
 		   ( (  (value) & 0x08 ) >> 2 ) |
 		   ( ( bytetemp & 0x08 ) >> 1 );
-	contend( HL, 1 ); contend( HL, 1 ); contend( HL, 1 ); contend( HL, 1 );
-	contend( HL, 1 );
+	contend_read( HL, 1 ); contend_read( HL, 1 ); contend_read( HL, 1 );
+	contend_read( HL, 1 ); contend_read( HL, 1 );
 	HL$modifier; BC--;
 	F = ( F & FLAG_C ) | ( BC ? ( FLAG_V | FLAG_N ) : FLAG_N ) |
 	  halfcarry_sub_table[lookup] | ( bytetemp ? 0 : FLAG_Z ) |
@@ -150,8 +152,8 @@ sub cpir_cpdr ($) {
 	if(F & FLAG_H) bytetemp--;
 	F |= ( bytetemp & FLAG_3 ) | ( (bytetemp&0x02) ? FLAG_5 : 0 );
 	if( ( F & ( FLAG_V | FLAG_Z ) ) == FLAG_V ) {
-	  contend( HL, 1 ); contend( HL, 1 ); contend( HL, 1 );
-	  contend( HL, 1 ); contend( HL, 1 );
+	  contend_read( HL, 1 ); contend_read( HL, 1 ); contend_read( HL, 1 );
+	  contend_read( HL, 1 ); contend_read( HL, 1 );
 	  PC-=2;
 	}
       }
@@ -172,18 +174,22 @@ sub inc_dec ($$) {
 	print << "CODE";
       {
 	libspectrum_byte bytetemp = readbyte( HL );
-	tstates++;
+	contend_read( HL, 1 );
 	$opcode(bytetemp);
 	writebyte(HL,bytetemp);
       }
 CODE
     } elsif( $arg eq '(REGISTER+dd)' ) {
 	print << "CODE";
-      tstates += 6;		/* FIXME: how is this contended? */
       {
-	libspectrum_word wordtemp =
-	    REGISTER + (libspectrum_signed_byte)readbyte( PC++ );
-	libspectrum_byte bytetemp = readbyte( wordtemp );
+	libspectrum_byte offset, bytetemp;
+	libspectrum_word wordtemp;
+	offset = readbyte( PC );
+	contend_read( PC, 1 ); contend_read( PC, 1 ); contend_read( PC, 1 );
+	contend_read( PC, 1 ); contend_read( PC, 1 ); PC++;
+	wordtemp = REGISTER + (libspectrum_signed_byte)offset;
+	bytetemp = readbyte( wordtemp );
+	contend_read( wordtemp, 1 );
 	$opcode(bytetemp);
 	writebyte(wordtemp,bytetemp);
       }
@@ -200,11 +206,15 @@ sub ini_ind ($) {
 
     print << "CODE";
       {
-	libspectrum_word initemp = readport( BC );
-	tstates += 2; contend_io( BC, 3 );
-	writebyte(HL,initemp);
+	libspectrum_word initemp;
+
+	tstates++;
+	ula_contend_port( BC );
+	initemp = readport( BC );
+	writebyte( HL, initemp );
+
 	B--; HL$modifier;
-	F = (initemp & 0x80 ? FLAG_N : 0 ) | sz53_table[B];
+	F = ( initemp & 0x80 ? FLAG_N : 0 ) | sz53_table[B];
 	/* C,H and P/V flags not implemented */
       }
 CODE
@@ -218,16 +228,22 @@ sub inir_indr ($) {
 
     print << "CODE";
       {
-	libspectrum_word initemp=readport( BC );
-	tstates += 2; contend_io( BC, 3 );
-	writebyte(HL,initemp);
+	libspectrum_word initemp;
+
+	tstates++;
+	ula_contend_port( BC );
+	initemp = readport( BC );
+	writebyte( HL, initemp );
+
 	B--; HL$modifier;
-	F = (initemp & 0x80 ? FLAG_N : 0 ) | sz53_table[B];
+	F = ( initemp & 0x80 ? FLAG_N : 0 ) | sz53_table[B];
 	/* C,H and P/V flags not implemented */
-	if(B) {
-	  contend( HL, 1 ); contend( HL, 1 ); contend( HL, 1 ); contend( HL, 1 );
-	  contend( HL, 1 );
-	  PC-=2;
+
+	if( B ) {
+	  contend_write( HL, 1 ); contend_write( HL, 1 );
+	  contend_write( HL, 1 ); contend_write( HL, 1 );
+	  contend_write( HL, 1 );
+	  PC -= 2;
 	}
       }
 CODE
@@ -245,7 +261,7 @@ sub ldi_ldd ($) {
 	libspectrum_byte bytetemp=readbyte( HL );
 	BC--;
 	writebyte(DE,bytetemp);
-	contend( DE, 1 ); contend( DE, 1 );
+	contend_write( DE, 1 ); contend_write( DE, 1 );
 	DE$modifier; HL$modifier;
 	bytetemp += A;
 	F = ( F & ( FLAG_C | FLAG_Z | FLAG_S ) ) | ( BC ? FLAG_V : 0 ) |
@@ -264,14 +280,15 @@ sub ldir_lddr ($) {
       {
 	libspectrum_byte bytetemp=readbyte( HL );
 	writebyte(DE,bytetemp);
-	contend( DE, 1 ); contend( DE, 1 );
+	contend_write( DE, 1 ); contend_write( DE, 1 );
 	HL$modifier; DE$modifier; BC--;
 	bytetemp += A;
 	F = ( F & ( FLAG_C | FLAG_Z | FLAG_S ) ) | ( BC ? FLAG_V : 0 ) |
 	  ( bytetemp & FLAG_3 ) | ( (bytetemp & 0x02) ? FLAG_5 : 0 );
 	if(BC) {
-	  contend( DE, 1 ); contend( DE, 1 ); contend( DE, 1 );
-	  contend( DE, 1 ); contend( DE, 1 );
+	  contend_write( DE, 1 ); contend_write( DE, 1 );
+	  contend_write( DE, 1 ); contend_write( DE, 1 );
+	  contend_write( DE, 1 );
 	  PC-=2;
 	}
       }
@@ -287,20 +304,21 @@ sub otir_otdr ($) {
     print << "CODE";
       {
 	libspectrum_word outitemp;
+
 	tstates++;
-	outitemp = readbyte( HL ); tstates++;
-	B--; HL$modifier; /* This does happen first, despite what the specs say */
+	outitemp = readbyte( HL );
+	B--;	/* This does happen first, despite what the specs say */
+	ula_contend_port( BC );
 	writeport(BC,outitemp);
+
+	HL$modifier;
 	F = (outitemp & 0x80 ? FLAG_N : 0 ) | sz53_table[B];
 	/* C,H and P/V flags not implemented */
-	if(B) {
-	  contend_io( BC, 1 );
-	  contend( PC, 1 ); contend( PC, 1 ); contend( PC, 1 );
-	  contend( PC, 1 ); contend( PC, 1 ); contend( PC, 1 );
-	  contend( PC - 1, 1 );
-	  PC-=2;
-	} else {
-	  contend_io( BC, 3 );
+
+	if( B ) {
+	  contend_read( HL, 1 ); contend_read( HL, 1 ); contend_read( HL, 1 );
+	  contend_read( HL, 1 ); contend_read( HL, 1 );
+	  PC -= 2;
 	}
       }
 CODE
@@ -315,12 +333,14 @@ sub outi_outd ($) {
     print << "CODE";
       {
 	libspectrum_word outitemp;
+
 	tstates++;
-	outitemp = readbyte( HL ); tstates++;
+	outitemp = readbyte( HL );
 	B--;	/* This does happen first, despite what the specs say */
-	contend_io( BC, 3 );
-	HL$modifier;
+	ula_contend_port( BC );
 	writeport(BC,outitemp);
+
+	HL$modifier;
 	F = (outitemp & 0x80 ? FLAG_N : 0 ) | sz53_table[B];
 	/* C,H and P/V flags not implemented */
       }
@@ -366,14 +386,18 @@ sub res_set ($$$) {
 	print << "CODE";
       {
 	libspectrum_byte bytetemp = readbyte( HL );
-	tstates++;
+	contend_read( HL, 1 );
 	writebyte( HL, bytetemp $operator $hex_mask );
       }
 CODE
     } elsif( $register eq '(REGISTER+dd)' ) {
 	print << "CODE";
-      tstates += 2;
-      writebyte(tempaddr, readbyte(tempaddr) $operator $hex_mask);
+      {
+	libspectrum_byte bytetemp;
+	bytetemp = readbyte( tempaddr );
+	contend_read( tempaddr, 1 );
+	writebyte( tempaddr, bytetemp $operator $hex_mask );
+      }
 CODE
     }
 }
@@ -388,16 +412,16 @@ sub rotate_shift ($$) {
 	print << "CODE";
       {
 	libspectrum_byte bytetemp = readbyte(HL);
-	tstates++;
+	contend_read( HL, 1 );
 	$opcode(bytetemp);
 	writebyte(HL,bytetemp);
       }
 CODE
     } elsif( $register eq '(REGISTER+dd)' ) {
 	print << "CODE";
-      tstates += 2;
       {
 	libspectrum_byte bytetemp = readbyte(tempaddr);
+	contend_read( tempaddr, 1 );
 	$opcode(bytetemp);
 	writebyte(tempaddr,bytetemp);
       }
@@ -417,24 +441,22 @@ sub opcode_BIT (@) {
 
     my( $bit, $register ) = @_;
 
-    my $macro = ( $bit == 7 ? '7(' : "($bit," );
-
     if( length $register == 1 ) {
-	print "      BIT$macro$register);\n";
+	print "      BIT( $bit, $register );\n";
     } elsif( $register eq '(REGISTER+dd)' ) {
 	print << "BIT";
-      tstates += 2;
       {
 	libspectrum_byte bytetemp = readbyte( tempaddr );
-	BIT${macro}bytetemp);
+	contend_read( tempaddr, 1 );
+	BIT( $bit, bytetemp );
       }
 BIT
     } else {
 	print << "BIT";
       {
 	libspectrum_byte bytetemp = readbyte( HL );
-	tstates++;
-	BIT${macro}bytetemp);
+	contend_read( HL, 1 );
+	BIT( $bit, bytetemp );
       }
 BIT
     }
@@ -496,7 +518,7 @@ sub opcode_DJNZ (@) {
       if(B) {
 	JR();
       } else {
-	contend( PC, 3 );
+	contend_read( PC, 3 );
       }
       PC++;
 DJNZ
@@ -544,8 +566,9 @@ EX
 	libspectrum_byte bytetempl, bytetemph;
 	bytetempl = readbyte( SP );
 	bytetemph = readbyte( SP + 1 ); tstates++;
-	writebyte( SP,    $low  );
-	writebyte( SP + 1,$high ); tstates += 2;
+	writebyte( SP,     $low  );
+	writebyte( SP + 1, $high );
+	contend_write( SP + 1, 1 ); contend_write( SP + 1, 1 );
 	$low=bytetempl; $high=bytetemph;
       }
 EX
@@ -586,14 +609,13 @@ sub opcode_IN (@) {
 	print << "IN";
       { 
 	libspectrum_word intemp;
-	intemp = readbyte( PC++ ) + ( A << 8 ); tstates++;
-	contend_io( intemp, 3 );
+	intemp = readbyte( PC++ ) + ( A << 8 );
+	ula_contend_port( intemp );
         A=readport( intemp );
       }
 IN
     } elsif( $register eq 'F' and $port eq '(C)' ) {
 	print << "IN";
-      tstates += 1;
       {
 	libspectrum_byte bytetemp;
 	Z80_IN( bytetemp, BC );
@@ -601,7 +623,6 @@ IN
 IN
     } elsif( length $register == 1 and $port eq '(C)' ) {
 	print << "IN";
-      tstates += 1;
       Z80_IN( $register, BC );
 IN
     }
@@ -648,7 +669,7 @@ sub opcode_JR (@) {
       if( $condition_string ) {
         JR();
       } else {
-        contend( PC, 3 );
+        contend_read( PC, 3 );
       }
 JR
     }
@@ -702,8 +723,13 @@ LD
 LD
         } elsif( $src eq '(REGISTER+dd)' ) {
 	    print << "LD";
-      tstates += 5;		/* FIXME: how is this contended? */
-      $dest = readbyte( REGISTER + (libspectrum_signed_byte)readbyte( PC++ ) );
+      {
+	libspectrum_byte offset;
+	offset = readbyte( PC );
+	contend_read( PC, 1 ); contend_read( PC, 1 ); contend_read( PC, 1 );
+	contend_read( PC, 1 ); contend_read( PC, 1 ); PC++;
+	$dest = readbyte( REGISTER + (libspectrum_signed_byte)offset );
+      }
 LD
         }
 
@@ -769,16 +795,22 @@ LD
 
 	if( length $src == 1 ) {
 	print << "LD";
-      tstates += 5;		/* FIXME: how is this contended? */
-      writebyte( REGISTER + (libspectrum_signed_byte)readbyte( PC++ ), $src );
+      {
+	libspectrum_byte offset;
+	offset = readbyte( PC );
+	contend_read( PC, 1 ); contend_read( PC, 1 ); contend_read( PC, 1 );
+	contend_read( PC, 1 ); contend_read( PC, 1 ); PC++;
+	writebyte( REGISTER + (libspectrum_signed_byte)offset, $src );
+      }
 LD
         } elsif( $src eq 'nn' ) {
 	    print << "LD";
-      tstates += 2;		/* FIXME: how is this contended? */
       {
-	libspectrum_word wordtemp =
-	    REGISTER + (libspectrum_signed_byte)readbyte( PC++ );
-	writebyte(wordtemp,readbyte(PC++));
+	libspectrum_byte offset, value;
+	offset = readbyte( PC++ );
+	value = readbyte( PC );
+	contend_read( PC, 1 ); contend_read( PC, 1 ); PC++;
+	writebyte( REGISTER + (libspectrum_signed_byte)offset, value );
       }
 LD
         }
@@ -821,13 +853,11 @@ sub opcode_OUT (@) {
       { 
 	libspectrum_word outtemp;
 	outtemp = readbyte( PC++ ) + ( A << 8 );
-	tstates++;
 	Z80_OUT( outtemp , A );
       }
 OUT
     } elsif( $port eq '(C)' and length $register == 1 ) {
 	print << "OUT";
-      tstates += 1;
       Z80_OUT( BC, $register );
 OUT
     }
@@ -910,7 +940,8 @@ sub opcode_RLD (@) {
     print << "RLD";
       {
 	libspectrum_byte bytetemp = readbyte( HL );
-	tstates += 4;
+	contend_read( HL, 1 ); contend_read( HL, 1 ); contend_read( HL, 1 );
+	contend_read( HL, 1 );
 	writebyte(HL, (bytetemp << 4 ) | ( A & 0x0f ) );
 	A = ( A & 0xf0 ) | ( bytetemp >> 4 );
 	F = ( F & FLAG_C ) | sz53p_table[A];
@@ -945,7 +976,8 @@ sub opcode_RRD (@) {
     print << "RRD";
       {
 	libspectrum_byte bytetemp = readbyte( HL );
-	tstates += 4;
+	contend_read( HL, 1 ); contend_read( HL, 1 ); contend_read( HL, 1 );
+	contend_read( HL, 1 );
 	writebyte(HL,  ( A << 4 ) | ( bytetemp >> 4 ) );
 	A = ( A & 0xf0 ) | ( bytetemp & 0x0f );
 	F = ( F & FLAG_C ) | sz53p_table[A];
@@ -964,8 +996,9 @@ sub opcode_SBC (@) { arithmetic_logical( 'SBC', $_[0], $_[1] ); }
 
 sub opcode_SCF (@) {
     print << "SCF";
-      F &= ~( FLAG_N | FLAG_H );
-      F |= ( A & ( FLAG_3 | FLAG_5 ) ) | FLAG_C;
+      F = ( F & ( FLAG_P | FLAG_Z | FLAG_S ) ) |
+	  ( A & ( FLAG_3 | FLAG_5          ) ) |
+	  FLAG_C;
 SCF
 }
 
@@ -984,16 +1017,7 @@ sub opcode_SUB (@) { arithmetic_logical( 'SUB', $_[0], $_[1] ); }
 sub opcode_XOR (@) { arithmetic_logical( 'XOR', $_[0], $_[1] ); }
 
 sub opcode_slttrap ($) {
-    print << "slttrap";
-      if( settings_current.slt_traps ) {
-	if( slt_length[A] ) {
-	  libspectrum_word base = HL;
-	  libspectrum_byte *data = slt[A];
-	  size_t length = slt_length[A];
-	  while( length-- ) writebyte( base++, *data++ );
-	}
-      }
-slttrap
+    print "      slt_trap( HL, A );\n";
 }
 
 sub opcode_shift (@) {
@@ -1005,14 +1029,14 @@ sub opcode_shift (@) {
     if( $opcode eq 'DDFDCB' ) {
 
 	print << "shift";
-      /* FIXME: contention here is just a guess */
       {
 	libspectrum_word tempaddr; libspectrum_byte opcode3;
-	contend( PC, 3 );
+	contend_read( PC, 3 );
 	tempaddr =
 	    REGISTER + (libspectrum_signed_byte)readbyte_internal( PC );
-	PC++; contend( PC, 4 );
-	opcode3 = readbyte_internal( PC ); PC++;
+	PC++; contend_read( PC, 3 );
+	opcode3 = readbyte_internal( PC );
+	contend_read( PC, 1 ); contend_read( PC, 1 ); PC++;
 #ifdef HAVE_ENOUGH_MEMORY
 	switch(opcode3) {
 #include "z80_ddfdcb.c"
@@ -1026,7 +1050,7 @@ shift
 	print << "shift";
       {
 	libspectrum_byte opcode2;
-	contend( PC, 4 );
+	contend_read( PC, 4 );
 	opcode2 = readbyte_internal( PC ); PC++;
 	R++;
 #ifdef HAVE_ENOUGH_MEMORY
@@ -1125,16 +1149,16 @@ while(<>) {
 	    my $hexmask = res_set_hexmask( $opcode, $bit );
 
 	    print << "CODE";
-      tstates += 2;
-      $register=readbyte(tempaddr) $operator $hexmask;
+      $register = readbyte(tempaddr) $operator $hexmask;
+      contend_read( tempaddr, 1 );
       writebyte(tempaddr, $register);
       break;
 CODE
 	} else {
 
 	    print << "CODE";
-      tstates += 2;
       $register=readbyte(tempaddr);
+      contend_read( tempaddr, 1 );
       $opcode($register);
       writebyte(tempaddr, $register);
       break;
