@@ -1,7 +1,7 @@
 /* z80_macros.h: Some commonly used z80 things as macros
    Copyright (c) 1999-2003 Philip Kendall
 
-   $Id: z80_macros.h,v 1.26.2.1 2004/01/14 19:59:05 pak21 Exp $
+   $Id: z80_macros.h,v 1.32 2004/06/09 10:55:09 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -102,19 +102,23 @@
 
 #ifndef CORETEST
 
-#define contend(address,time) \
-  if( memory_map[ (address) >> 13 ].contended ) \
-    tstates += spectrum_contention[ tstates ]; \
+#define contend_read(address,time) \
+  if( memory_map_read[ (address) >> 13 ].contended ) \
+    tstates += ula_contention[ tstates ]; \
+  tstates += (time);
+
+#define contend_write(address,time) \
+  if( memory_map_write[ (address) >> 13 ].contended ) \
+    tstates += ula_contention[ tstates ]; \
   tstates += (time);
 
 #else				/* #ifndef CORETEST */
 
-void contend( libspectrum_word address, libspectrum_dword time );
+void contend_read( libspectrum_word address, libspectrum_dword time );
+void contend_write( libspectrum_word address, libspectrum_dword time );
 
 #endif				/* #ifndef CORETEST */
 
-#define contend_io(port,time) tstates += contend_port( (port) ) + (time);
-
 /* Some commonly used instructions */
 #define AND(value)\
 {\
@@ -174,17 +178,15 @@ void contend( libspectrum_word address, libspectrum_dword time );
     halfcarry_add_table[lookup];\
 }
 
-#define BIT(bit,value)\
-{\
-  F = ( F & FLAG_C ) | ( (value) & ( FLAG_3 | FLAG_5 ) ) |\
-    ( ( (value) & ( 0x01 << bit ) ) ? FLAG_H : ( FLAG_P | FLAG_H | FLAG_Z ) );\
-}
-
-#define BIT7(value)\
-{\
-  F = ( F & FLAG_C ) | ( (value) & ( FLAG_3 | FLAG_5 ) ) |\
-    ( ( (value) & 0x80 ) ? ( FLAG_H | FLAG_S ) :\
-      ( FLAG_P | FLAG_H | FLAG_Z ) );\
+/* This may look fairly inefficient, but the (gcc) optimiser does the
+   right thing assuming it's given a constant for 'bit' */
+#define BIT( bit, value ) \
+{ \
+  F = ( F & FLAG_C ) | FLAG_H; \
+  if( ! ( (value) & ( 0x01 << (bit) ) ) ) F |= FLAG_P | FLAG_Z; \
+  if( (bit) == 3 && (value) & 0x08 ) F |= FLAG_3; \
+  if( (bit) == 5 && (value) & 0x20 ) F |= FLAG_5; \
+  if( (bit) == 7 && (value) & 0x80 ) F |= FLAG_S; \
 }
 
 #define CALL()\
@@ -192,7 +194,7 @@ void contend( libspectrum_word address, libspectrum_dword time );
   libspectrum_byte calltempl, calltemph; \
   calltempl=readbyte(PC++);\
   calltemph=readbyte( PC ); \
-  contend( PC, 1 ); PC++;\
+  contend_read( PC, 1 ); PC++;\
   PUSH16(PCL,PCH);\
   PCL=calltempl; PCH=calltemph;\
 }
@@ -229,7 +231,7 @@ break
 
 #define Z80_IN( reg, port )\
 {\
-  contend_io( port, 3 );\
+  ula_contend_port( port ); \
   (reg)=readport((port));\
   F = ( F & FLAG_C) | sz53p_table[(reg)];\
 }
@@ -271,8 +273,8 @@ break
 #define JR()\
 {\
   libspectrum_signed_byte jrtemp = readbyte( PC ); \
-  contend( PC, 1 ); contend( PC, 1 ); contend( PC, 1 ); contend( PC, 1 );\
-  contend( PC, 1 );\
+  contend_read( PC, 1 ); contend_read( PC, 1 ); contend_read( PC, 1 ); \
+  contend_read( PC, 1 ); contend_read( PC, 1 ); \
   PC += jrtemp; \
 }
 
@@ -284,7 +286,7 @@ break
 
 #define Z80_OUT( port, reg )\
 {\
-  contend_io( port, 3 );\
+  ula_contend_port( port ); \
   writeport(port,reg);\
 }
 
