@@ -1,7 +1,7 @@
 /* z80_ops.c: Process the next opcode
-   Copyright (c) 1999-2003 Philip Kendall, Witold Filipczyk
+   Copyright (c) 1999-2004 Philip Kendall, Witold Filipczyk
 
-   $Id: z80_ops.c,v 1.48 2003/12/31 00:17:20 pak21 Exp $
+   $Id: z80_ops.c,v 1.56 2004/06/09 10:55:09 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,11 +32,13 @@
 #include "event.h"
 #include "machine.h"
 #include "memory.h"
+#include "periph.h"
 #include "rzx.h"
 #include "settings.h"
-#include "spectrum.h"
+#include "slt.h"
 #include "tape.h"
 #include "trdos.h"
+#include "ula.h"
 #include "z80.h"
 
 #include "z80_macros.h"
@@ -88,16 +90,26 @@ static void z80_ddfdcbxx( libspectrum_byte opcode3,
 void
 z80_do_opcodes( void )
 {
+  int even_m1;
+  void *cgoto[4]; size_t next;
+
+  /* Avoid 'variable not used' warnings if we're not using gcc */
+  cgoto[0] = cgoto[0]; next = next;
+
+  even_m1 =
+    machine_current->capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_EVEN_M1; 
 
 #ifdef __GNUC__
 
-  void *cgoto[3]; size_t next = 0;
-
+  next = 0;
   SETUP_CHECK( rzx, rzx_playback, 0 );
   SETUP_CHECK( debugger, debugger_mode != DEBUGGER_MODE_INACTIVE, 1 );
   SETUP_CHECK( trdos, trdos_available, 2 );
+  if( next != 3 ) { cgoto[ next ] = &&opcode_delay; }
 
-  if( next != 3 ) { cgoto[ next ] = &&run_opcode; }
+  next = 3;
+  SETUP_CHECK( evenm1, even_m1, 3 );
+  if( next != 4 ) { cgoto[ next ] = &&run_opcode; }
 
 #endif				/* #ifdef __GNUC__ */
 
@@ -127,27 +139,33 @@ z80_do_opcodes( void )
     CHECK( trdos, trdos_available, 2 )
 
     if( trdos_active ) {
-      if( machine_current->ram.current_rom == 1 &&
+      if( machine_current->ram.current_rom &&
 	  PC >= 16384 ) {
-	trdos_active = 0;
-	memory_map[0].page = &ROM[ machine_current->ram.current_rom ][0x0000];
-	memory_map[1].page = &ROM[ machine_current->ram.current_rom ][0x2000];
+	trdos_unpage();
       }
     } else if( ( PC & 0xff00 ) == 0x3d00 &&
-	       machine_current->ram.current_rom == 1 ) {
-      trdos_active = 1;
-      memory_map[0].page = &ROM[2][0x0000];
-      memory_map[1].page = &ROM[2][0x2000];
+	       machine_current->ram.current_rom ) {
+      trdos_page();
     }
 
     END_CHECK
 
+  opcode_delay:
+
+    contend_read( PC, 4 );
+
+    /* Check to see if M1 cycles happen on even tstates */
+    CHECK( evenm1, even_m1, 3 )
+
+    if( tstates & 1 ) tstates++;
+
+    END_CHECK
+
   run_opcode:
 
     /* Do the instruction fetch; readbyte_internal used here to avoid
        triggering read breakpoints */
-    contend( PC, 4 ); R++;
-    opcode = readbyte_internal( PC ); PC++;
+    opcode = readbyte_internal( PC ); PC++; R++;
 
     switch(opcode) {
 #include "opcodes_base.c"
